/*
Name of file:
   FileTimes.cc

Description:

   This class contains functions to set and get the specific date and time at which
   a given file was created, last accessed, or last written to.

   From Windows API:

   Windows records the times using a 64-bit value specifying the number of 100-nanosecond
   intervals that have elapsed since 12:00 A.M. January 1, 1601 (UTC).

   Notice: UTC time more or less the same as GMT.

   Writing to a file changes the last write time; writing to or reading from the file
   (including running an executable file) changes the last access time.

Programmer:
   Ivar B. Jessen

History:
   November 2005     Original version  of class
   19. June 2009     Thorough clean up of code
                     Fixed bugs
                     Added comments
                     Fixed bugs found by Mervyn Bick
                     Opened file as 'Read' in get-methods
                     Changed CPTR to CHANDLE in API functions
3. November 2009     Added methods SetSystemTime() and SetLocalTime()

Dependencies:
   windef.h
   winbase.h
   structAPI.h	
   structure.prg in Samples folder

Instantiation:
   set procedure to FileTimes.cc additive
   ft = new FileTimes()

Usage:
   ft.setDateCreated( cFilename, {20-06-2009 20:06:09} )   	
   ? ft.getDateCreated( cFilename ) // result 20-06-2009 20:06:09
   The result is of type DT
   
   Notice that the time markers will be set according to the language set in
   Control Panel -> Regional settings.
   An Italian user will see the above result as: 20/06/2009 20.06.09

Methods:

   GetSystemTime()  Retrieves the current system date and time.
                     The system time is expressed in Coordinated Universal Time (UTC). 
   SetSystemTime()   Sets the current system date and time. (UTC)
   GetLocalTime()   Retrieves the current local date and time. 
   SetLocalTime()   Sets the current local date and time. 

   Methods with UTC in the name sets and gets time as UTC time
   Methods without UTC in the name sets and gets time as Local time 

   No explanations given for the following methods as the purpose of each method
   should be clear from it's name.

   Methods retrieving a files date and time settings:

   getUTCDateCreated()		
   getUTCDateLastAccessed()
   getUTCDateLastModified()

   getDateCreated()
   getDateLastAccessed()
   getDateLastModified()

   Methods setting a files date and time:

   setUTCDateCreated()
   setUTCDateLastAccessed()
   setUTCDateLastModified()

   setDatCreated()
   setDateLastAccessed()
   setDateLastModified()

*/

Class FileTimes

protect sTime, lfTime, fTime, filename
protect Init, GetStValues, SetStValues

this.Name = "FileTimes.cc"
this.filename = space(256)

set proc to :samples:structure.prg additive

this.sTime = new SYSTEMTIME()
this.fTime = new FILETIME()
this.lfTime = new LFILETIME()

this.init()


function GetSystemTime
********************
   // fill Systemtime structure with values
   GetSystemTime(this.sTime.value)

   return class::getSTvalues(this.sTime)


function GetLocalTime
********************
   // fill Systemtime structure with values
   GetLocalTime(this.sTime.value)

   return class::getSTvalues(this.sTime)


function SetSystemTime
********************
parameter dDate

   // fill Systemtime structure with values
   SetSystemTime(this.sTime.value)

   // fill dDate into Systemtime structure
   class::setSTvalues(this.sTime, dtodt(dDate))


   function SetLocalTime
********************
parameter dDate

   // fill Systemtime structure with values
   SetLocalTime(this.sTime.value)

   // fill dDate into Systemtime structure
   class::setSTvalues(this.sTime, dtodt(dDate))



********** GET date...  starts here ****************


function getUTCDateCreated
***************************
   parameter cFilename
   local f

   // get Date Created in UTC time
   // get file handle
   f = new file()
   f.open(cFilename, "R")

   // fill filetime structure
   GetFileTime(f.handle, this.fTime.value, null, null)

   // close file
   f.close()

   // move data from FileTime structure to SystemTime structure
   FileTimeToSystemTime(this.fTime.value, this.sTime.value)

   return class::getSTvalues(this.sTime.value)


function getDateCreated
************************
   parameter cFilename
   local f

   // get Date Created in Local time
   // get file handle
   f = new file()
   f.open(cFilename, "R")

   // fill filetime structure
   GetFileTime(f.handle, this.fTime.value, null, null)

   // close file
   f.close()

   // move data from FileTime structure to LocalFiletime structure
   FileTimeToLocalFileTime(this.fTime.value, this.lfTime.value)

   // move data from  LocalFileTime structure to SystemTime structure
   FileTimeToSystemTime(this.lfTime.value, this.sTime.value)

   return class::getSTvalues(this.sTime.value)


function getUTCDateLastAccessed
********************************
   parameter cFilename
   local f

   // get Date Last Modified in UTC time
   // get file handle
   f = new file()
   f.open(cFilename, "R")

   // fill filetime structure
   GetFileTime(f.handle, null, this.fTime.value, null)

   // close file
   f.close()

   // move data from FileTime structure to SystemTime structure
   FileTimeToSystemTime(this.fTime.value, this.sTime.value)

   return class::getSTvalues(this.sTime.value)


function getDateLastAccessed
*****************************
   parameter cFilename
   local f

   // get Date last Modified in Local time
   // get file handle
   f = new file()
   f.open(cFilename, "R")

   // fill filetime structure
   GetFileTime(f.handle, null, this.fTime.value, null)

   // close file
   f.close()

   // move data from FileTime structure to LocalFiletime structure
   FileTimeToLocalFileTime(this.fTime.value, this.lfTime.value)

   // move data from  LocalFileTime structure to SystemTime structure
   FileTimeToSystemTime(this.lfTime.value, this.sTime.value)

   return class::getSTvalues(this.sTime.value)

   
function getUTCDateLastModified
********************************
   parameter cFilename
   local f

   // get Date Last Modified in UTC time
   // get file handle
   f = new file()
   f.open(cFilename, "R")

   // fill filetime structure
   GetFileTime(f.handle, null, null, this.fTime.value)

   // close file
   f.close()

   // move data from FileTime structure to SystemTime structure
   FileTimeToSystemTime(this.fTime.value, this.sTime.value)

   return class::getSTvalues(this.sTime.value)


function getDateLastModified
*****************************
   parameter cFilename
   local f

   // get Date last Modified in Local time
   // get file handle
   f = new file()
   f.open(cFilename, "R")

   // fill filetime structure
   GetFileTime(f.handle, null, null, this.fTime.value)

   // close file
   f.close()

   // move data from FileTime structure to LocalFiletime structure
   FileTimeToLocalFileTime(this.fTime.value, this.lfTime.value)

   // move data from  LocalFileTime structure to SystemTime structure
   FileTimeToSystemTime(this.lfTime.value, this.sTime.value)

   return class::getSTvalues(this.sTime.value)


************* SET date...  starts here ********************


function setUTCDateCreated
***************************
   parameter cFilename, dDate
   local f

   // set Date Created in UTC time
   // get file handle
   f = new file()
   f.open(cFilename, "W")

   // fill dDate into Systemtime structure
   class::setSTvalues(this.sTime, dtodt(dDate))

   // move data from Sytemtime structure to Filetime structure
   SystemTimeToFileTime(this.sTime.value, this.fTime.value) 

   // set date into file
   SetFileTime(f.handle, this.fTime.value, null, null)

   // close file
   f.close()
   return 
   

function setDateCreated
************************
   parameter cFilename, dDate
   local f

   // set Date Created in Local time
   // get file handle
   f = new file()
   f.open(cFilename, "W")

   // fill dDate into Systemtime structure 
   class::setSTvalues(this.sTime, dtodt(dDate))

   // move data from Sytemtime structure to Local Filetime structure
   SystemTimeToFileTime(this.sTime.value, this.lfTime.value) 

   // move data from Local FileTime structure to Filetime structure
   LocalFileTimeToFileTime(this.lfTime.value, this.fTime.value)

   // set date into file
   SetFileTime(f.handle, this.fTime.value, null, null)

   // close file
   f.close()
   return 


function setUTCDateLastAccessed
********************************
   parameter cFilename, dDate
   local f

   // set Date Last Accessed in UTC time
   // get file handle
   f = new file()
   f.open(cFilename, "W")

   // fill dDate into Systemtime structure
   class::setSTvalues(this.sTime, dtodt(dDate))

   // move data from Sytemtime structure to Filetime structure
   SystemTimeToFileTime(this.sTime.value, this.fTime.value) 

   SetFileTime(f.handle, null, this.fTime.value, null)

   // close file
   f.close()
   return 


function setDateLastAccessed
*****************************
   parameter cFilename, dDate
   local f

   // set Date Last Accessed in Local time
   // get file handle
   f = new file()
   f.open(cFilename, "W")

   // fill dDate into Systemtime structure 
   class::setSTvalues(this.sTime, dtodt(dDate))

   // move data from Sytemtime structure to Local Filetime structure
   SystemTimeToFileTime(this.sTime.value, this.lfTime.value) 

   // move data from Local FileTime structure to Filetime structure
   LocalFileTimeToFileTime(this.lfTime.value, this.fTime.value)

   // set date into file
   SetFileTime(f.handle, null, this.fTime.value, null)

   // close file
   f.close()
   return 

   
function setUTCDateLastModified
********************************
   parameter cFilename, dDate
   local f

   // set Date Last Modified in UTC time
   // get file handle
   f = new file()
   f.open(cFilename, "W")

   // fill dDate into Systemtime structure
   class::setSTvalues(this.sTime, dtodt(dDate))

   // move data from Sytemtime structure to Filetime structure
   SystemTimeToFileTime(this.sTime.value, this.fTime.value) 

   SetFileTime(f.handle, null, null, this.fTime.value)

   // close file
   f.close()
   return 


function setDateLastModified
*****************************
   parameter cFilename, dDate
   local f

   // set Date Last Modified in Local time
   // get file handle
   f = new file()
   f.open(cFilename, "W")

   // fill dDate into Systemtime structure 
   class::setSTvalues(this.sTime, dtodt(dDate))

   // move data from Sytemtime structure to Local Filetime structure
   SystemTimeToFileTime(this.sTime.value, this.lfTime.value) 

   // move data from Local FileTime structure to Filetime structure
   LocalFileTimeToFileTime(this.lfTime.value, this.fTime.value)

   // set date into file
   SetFileTime(f.handle, null, null, this.fTime.value)

   // close file
   f.close()
   return 


function init
***************

#include windef.h
#include winbase.h
#include structAPI.h


if type("GetSystemTime") # "FP"
	extern CVOID GetSystemTime( CPTR ) kernel32
endif

if type("GetLocalTime") # "FP"
	extern CVOID GetLocalTime( CPTR ) kernel32
endif

if type("SetSystemTime") # "FP"
	extern CLOGICAL SetSystemTime( CPTR ) kernel32
endif

if type("SetLocalTime") # "FP"
	extern CLOGICAL SetLocalTime( CPTR ) kernel32
endif

if type("GetFileTime") # "FP"
	extern CLOGICAL GetFileTime(CHANDLE, CPTR, CPTR, CPTR) kernel32
endif

if type("SetFileTime") # "FP"
	extern CLOGICAL SetFileTime(CHANDLE, CPTR, CPTR, CPTR) kernel32
endif

if type("FileTimeToSystemTime") # "FP"
	extern CLOGICAL FileTimeToSystemTime(CPTR, CPTR) kernel32
endif

if type("SystemTimeToFileTime") # "FP"
	extern CLOGICAL SystemTimeToFileTime(CPTR, CPTR) kernel32
endif

if type("FileTimeToLocalFileTime") # "FP"
	extern CLOGICAL FileTimeToLocalFileTime(CPTR, CPTR) kernel32
endif
         
if type("LocalFileTimeToFileTime") # "FP"
	extern CLOGICAL LocalFileTimeToFileTime(CPTR, CPTR) kernel32
endif

return

function setSTvalues(Tobj, sdd)
***********************************
	Tobj.setmember("wYear", Year(sdd))
	Tobj.setmember("wMonth", Month(sdd))
	Tobj.setmember("wDayOfWeek", 0)
	Tobj.setmember("wDay", Day(sdd))
	Tobj.setmember("wHour", Hour(sdd))
	Tobj.setmember("wMinute", Minute(sdd))
	Tobj.setmember("wSecond", Second(sdd))
	Tobj.setmember("wMilliseconds", 0)

	return


function getSTvalues(sTime)
********************************
dd = new date(this.sTime.getmember('wYear'), ;
              this.sTime.getmember('wMonth') - 1, ;
              this.sTime.getmember('wDay'), ;
              this.sTime.getmember('wHour'), ;
              this.sTime.getmember('wMinute'), ;
              this.sTime.getmember('wSecond'), ;
              this.sTime.getmember('wMilliseconds')) 
   return dtodt(dd) 

endclass

class FILETIME of structure
	super::addMember( TYPE_DWORD, 'dwLowDateTime')
	super::addMember( TYPE_DWORD, 'dwHighDateTime')
endclass

class LFILETIME of structure
	super::addMember( TYPE_DWORD, 'dwLowDateTime')
	super::addMember( TYPE_DWORD, 'dwHighDateTime')
endclass

class SYSTEMTIME of structure
	super::addMember( TYPE_WORD, 'wYear')
	super::addMember( TYPE_WORD, 'wMonth')
	super::addMember( TYPE_WORD, 'wDayOfWeek')
	super::addMember( TYPE_WORD, 'wDay')
	super::addMember( TYPE_WORD, 'wHour')
	super::addMember( TYPE_WORD, 'wMinute')
	super::addMember( TYPE_WORD, 'wSecond')
	super::addMember( TYPE_WORD, 'wMilliseconds')
endclass

