/*
   -----------------------------------------------------------------------------
   Programmers..: Gaetano De Luisi, Marc VdB, Mervyn Bick, Jay Parsons
   Version......: 1.0
   Date.........: January 2021
   Notes........: 
      A big thank you to Marc Vanden Berghen who did the heavy lifting on 
      API calls for this class.

      A class that provides information on planet Earth, timezones, 
      coordinates...etc.

      It started as a bug fix for issue #3094 (getTimeZoneOffset() does
      not always return the correct value) and grew into a set of methods 
      to calculate sunrise and sunset time for a given longitude/latitude,
      distance between two sets of coordinates and more.

      Issue #3094 affects only a few places like Australia (New South Wales and
      South Australia), Fiji and Samoa.

      // !!!!!! WARNING !!!!!!

      This CLASS works only if the timezone passed to the setdate method is in
      English! In my eyes, this is a bug in the windows operating system.
      All the timezone information is stored in the registry, but as a locale
      string. Problem here is that in some cases, different time zones have the
      same locale designation F.i. on my german system,

      Cen. Australia Standard Time  and
      Aus Central W. Standard Time

      have the same locale designation : Zentralaustralische Normalzeit. Only in 
      English, all the timezones can be identified without ambiguity.

      Default value is 0,0 in this case the isnight will reflect the situation
      at 0 latitude and 0 longitude for the time given in setdate

      If a timezone cannot be identified, UTC will be used as default and 
      an error code will be generated
   Usage........: set procedure to GeoTools.cc
                  oGeo = new geotools()
                  oGeo.setgeoloc(-34.9285,138.6007) -- Default are 0,0, off the coast of Guinea
                  oGeo.setdate("2020-12-09 20:06","Cen. Australia Standard Time")
                  ? oGeo.sunrise -- pre-requisite: use setGeoloc(), returns a local DST time
                  ? oGeo.sunset -- pre-requisite: use setGeoloc(), returns a local DST time
                  ? oGeo.isnight -- pre-requisite: use setGeoloc()
                  ? oGeo.hasDST -- tells you whether the supplied time zone uses daylight savings
                  ? oGeo.isDST -- tells you if DST in effect for the supplied time zone

      The Class also returns the actual local timeoffset to GMT based on the timezone used
      in windows

                     ? oGeo.aloffset

      Always check for errors, after running a function. If after a method call, 
      the oGeo.error.code is not 0 some error occured. The error information can be 
      found in oGeo.error.code and oGeo.error.message if a message exists.
   -----------------------------------------------------------------------------
*/

Class GeoTools

	this.error     = new GTException()		// for errorhandling
	this.tmzns     = new assocarray()		// stores all possible timezones
	this.tmzkey	   = "UTC"			// timezone name in English
	this.altmzkey	= ""					// name of local timezone
	this.tz_id     = 121				// timezone id start-value UTC
	this.tmstmp		= ""					// of form YYYY-MM-DD hh:mm:ss

	this.year 		= 0
	this.month		= 0
	this.date		= 0
	this.dow			= 0
	this.hour		= 0
	this.minute		= 0
	this.second		= 0

	this.tmz_info	 = replicate(chr(0),256)			// buffer for timezoneinfo struct
	this.tmz_alinfo = replicate(chr(0),256)		// buffer for actual local tz_info

	this.isnight	= false				// automatically set in  set_date
	this.offset 	= 0						// offset in minutes
	this.alOffset	= 0 					// actual local offset
	this.alDST		= false				// true, if actual local DST is set
	this.hasDST		= false				// true if chosen timezone has DST
	this.isDST		= false				//true if DST currently in effect

	this.lat			= 0						// latitude for sunrise calculation
	this.long		= 0						// longitude
	this.sunrise	= "00:00"		// sunrise time for tmz and date
	this.sunset		= "00:00"		// sunset
	this.tOffset   = 0
	this.tzByte0   = ""
	this.tzByte1   = ""
	this.init() 	// externalize needed windows APIs and fill the array of 
						// all timezone info: this.tmzns

	function SetGeoloc(lat,long)
		// some functions have return values other than logical, hence it is 
		// advisable to test for the error code instead of a the logical return value.
		// Clearing any error codes from previous runs
		this.error.clear( )
		if  (lat>90 or lat<-90) or (long>180 or long<-180)
			this.error.message = "A latitude and/or longitude is out of range"
			this.error.code = 1
			return
		endif		
		this.lat  = lat
		this.long = long
	return

	function SetDate(tmstmp,tmzn)	
		local d,t,result,systemtime,localtime,tmp

		this.error.clear()
		// if the timezone name is not supplied, use the local computer's timezone
		if empty (tmzn)
			tmzn = this.altmzkey		// local timezone, from init()
		endif
		d = ltrim(rtrim(tmzn))
		this.tmstmp = tmstmp

		try
			// Get the time_zone_information structure with the tz_id
			if not this.tmzns.iskey(d)
				this.error.code = 2
				this.error.message = "Timezone is unknown"	
				throw this.error
			endif

			this.tmzkey	= d
			this.tz_id			= this.tmzns[d]
			this.tmz_info = replicate(chr(0),256)
			result 				= EnumDynamicTimeZoneInformation(this.tz_id,this.tmz_info)			
			if result <> 0
				this.error.code			= getlasterror()
				this.error.message = "API error! Could not get the timezone information struct"
				// fall back to default values
				this.tz_id			= 121
				this.tmzkey	= "UTC"
				throw this.error
			endif

			if empty(tmstmp)				// no date passed, initialize with today
				d = new date()
				this.year 			= d.year
				this.month		= d.month+1
				this.date			= d.date
				this.dow			= d.day
				this.hour			= d.hour
				this.minute	= d.minute
				this.second	= d.second
				this.tmstmp	= this.year+"-"+this.month+"-"+this.date+" "+this.hour+":"+this.minute
			else
				this.year			= val(left(tmstmp,4))
				this.month		= val(substr(tmstmp,6,2))
				this.date			= val(substr(tmstmp,9,2))
				this.dow	   	= new date(this.year,this.month-1,this.date).getday()
				this.hour			= val(substr(tmstmp,12,2))
				this.minute	= val(substr(tmstmp,15,2))
				this.second	= val(substr(tmstmp,18,2))
			endif

			// get the offset for the specified date and timezone

			localtime = replicate(chr(0),16)
			systemtime = replicate(chr(0),16)
			tmp = itoh(this.year,4)
			localtime.setbyte(1,htoi(left(tmp,2)))		// fill the systemtime structure
			localtime.setbyte(0,htoi(right(tmp,2)))	// seconds are of no concern
			localtime.setbyte(2,this.month)
			localtime.setbyte(4,this.dow)
			localtime.setbyte(6,this.date)
			localtime.setbyte(8,this.hour)
			localtime.setbyte(10,this.minute)

			if TzSpecificLocalTimeToSystemTimeEx(this.tmz_info,localtime,systemtime)= false
				this.error.code = getlasterror()
				this.error.message = "API Error: Could not calculate the timezone offset"
				throw this.error
			endif
			tmp = sign(this.date-systemtime.getbyte(6))*24*60  //consider day boundary			
			if (this.date = 1 or systemtime.getbyte(6) = 1) ;
				and (this.month <> systemtime.getbyte(2))
				tmp = -tmp		// adjust for month boundaries
			endif
			tmp += (this.hour - systemtime.getbyte(8))*60			
			this.offset = tmp + this.minute - systemtime.getbyte(10)

			this.getsun()	// get the sunrise and sunset times for the supplied date
			t=substr(tmstmp,12,5)
			this.isnight = (t<this.sunrise) or (t>this.sunset)

		//determine if DST is applicable to the time zone provided
		this.hasDst = (this.tmz_info.getbyte(154) <> 0 )
		
		//determine if DST is currently active for the supplied timezone
		//htoi(right(tOffset,2)+left(tOffset,2) )
		tmp = this.tmz_info.getbyte(0)+256*this.tmz_info.getbyte(1)    // might be negative -> greater than 2^15
		tmp = iif(tmp > 2^15 , 2^16 - tmp , tmp)

		this.isDST = (abs(tmp) <> abs(this.offset))
		
		catch(exception e)
			return false
		endtry
	return true

	function Init
		extern culong GetLastError(cvoid) kernel32
		extern culong EnumDynamicTimeZoneInformation(culong,cptr) advapi32.dll
		extern clogical TzSpecificLocalTimeToSystemTimeEx(cptr,cptr,cptr) kernel32
		extern culong GetTimeZoneInformation(cptr) kernel32
		extern culong GetDynamicTimeZoneInformation(cptr) kernel32
		extern culong GetDynamicTimeZoneInformationEffectiveYears(cptr,cptr culong,cptr culong) advapi32.dll
		extern clogical GetTimeZoneInformationForYear(cushort,cptr,cptr) kernel32

		// fill the tmzns array with all possible values  rather than doing a loop
		// through the registry keys each time set_date is called, it makes
		// more sense to initialize an array and rely on it later on

		local i,result,buffer,std
		i = 0 ; result = 0
		do while result = 0
			buffer	= replicate(chr(0),256)
			result	= EnumDynamicTimeZoneInformation(i,buffer)
			std			= substr(buffer,87,128)			
			this.tmzns[left(std,at(chr(0),std))] = i
			i++
		enddo

		if result<> 259
			this.error.code			= result
			this.error.message	= "API-ERROR : "+itoh(result)+" occured."
			return false
		endif

		// get the actual (at this date and time) local (as set in windows)
		// timezone information

		local result,bias,dlt
		try
			result = GetDynamicTimeZoneInformation(this.tmz_alinfo)
			if result > 3
				this.error.code    = getlasterror()
				this.error.message = "Something went wrong retrieving the local tzi"
				throw this.error
			endif
			std = substr(this.tmz_alinfo,87,128)
			this.altmzkey = left(std,at(chr(0),std))
			bias = this.tmz_alinfo.getbyte(0)+256*this.tmz_alinfo.getbyte(1)
			if result = 0 					// actual local DLST disabled or not used
				this.aloffset = bias
				this.aldlst = false
				return true
			endif

			this.aldlst		= iif(result = 1, false, true)	//  1->STD else DLST
			dlt						= iif(this.aldlst,168,84)
			this.al_offset = bias + this.tmz_alinfo.getbyte(dlt)+256*this.tmz_alinfo.getbyte(dlt+1)

		catch (exception e)
			this.altmzkey = "Unknown"
			return false
		endtry

	return true

	/*
	Date:       2020-12-31
 
   Author:     Gaetano De Luisi,  Mervyn Bick and Jay Parsons
 
   Description: 
      A function to calculate sunrise and sunset for a given location on a given day.  The
      time is returned as a character string in the form hh:mm
      
      The function is based to a large extent on formulae used in a spreadsheet published 
      by the American National Oceanic and Atmospheric Administration.  The spreadsheet
      included calculations for every 6 minute interval in 24 hours.  The sunrise and sunset
      times varied slightly (less than 1 minute) over the 24 hours depending on the time used 
      in the calculation.  No explanation was given as to why there was a discrepancy and 
      which value was the actual time of sunrise or sunset.   For this function noon has been
      used in the calculation.  The calculations are based on Julian days and the in the spreadsheet
      fractions of  a day were added based on the time of day for the calculation.  This function
      uses the Julian day as calculated using Dateex.cc in the dUFLP.      
      
   Usage:

      The following arguments must be provided.
      
         lat   Latitude in degrees.  + values for Northern hemisphere and - values for Southern hemisphere.
                Accuracy to 2 decimal places should be adequate.
					 
         long  Longitude in degrees.  + values for Eastern hemisphere and - values for Western hemisphere.
                Accuracy to 2 decimal places should be adequate.
                
         timezone    The number of hours different from UTC.   + values to the East and - values to the West

         bRise   True to return sunrise or false to return sunset.
   */	

	function GetSun
      noon 	= 12
		d 		= new date(this.year,this.month-1,this.date)
		jday 	= 1721426 + d - ctod("001/001/001")	// by Jay Parsons //F Julian day
      jCent=(jDay-2451545)/36525		//G Julian century
      //I Geom Mean Long Sun (deg)
      GeoMeanLongSun=MOD(280.46646+jCent*(36000.76983 +jCent*0.0003032),360)
      //J Geom Mean Anom Sun (deg)
      GeoMeanAnomSun=357.52911+jCent*(35999.05029 - 0.0001537*jCent)
      //K Eccent Earth Orbit
      EccentEarth=0.016708634-jCent*(0.000042037+0.0000001267*jCent)
      //L Sun Eq of Ctr
      sunEqCtr=SIN(DTOR(GeoMeanAnomSun))*(1.914602-jCent*(0.004817+0.000014*jCent))+ ;
					SIN(DTOR(2*GeoMeanAnomSun))*(0.019993-0.000101*jCent)+ ;
					SIN(DTOR(3*GeoMeanAnomSun))*0.000289
      //M Sun True Long (deg)
      sunTrueLong=GeoMeanLongSun+sunEqCtr
      //N Sun True Anom (deg)
      sunTrueAnom=GeoMeanAnomSun+sunEqCtr
      //O Sun Rad Vector (AUs)
      sunRadVector=(1.000001018*(1-EccentEarth*EccentEarth))/(1+EccentEarth*COS(DTOR(sunTrueAnom)))
      //P Sun App Long (deg)
      sunAppLong=sunTrueLong-0.00569-0.00478*SIN(DTOR(125.04-1934.136*jCent))
      //QMean Obliq Ecliptic (deg)
      meanObliqEcliptic=23+(26+((21.448-jCent*(46.815+jCent*(0.00059-jCent*0.001813))))/60)/60
      //R Obliq Corr (deg)
      ObliqCorr=meanObliqEcliptic+0.00256*COS(DTOR(125.04-1934.136*jCent))
      //S Sun Rt Ascen (deg)
      SunRtAsc=RTOD(ATN2(COS(DTOR(sunAppLong)),COS(DTOR(ObliqCorr))*SIN(DTOR(sunAppLong))))
      //T Sun Declin (deg)
      sunDecl=RTOD(ASIN(SIN(DTOR(ObliqCorr))*SIN(DTOR(sunAppLong))))
      //U var y
      VarY=TAN(DTOR(ObliqCorr/2))*TAN(DTOR(ObliqCorr/2))
      //V Eq of Time (minutes)
      eqTime=4*RTOD(VarY*SIN(2*DTOR(GeoMeanLongSun))-2*EccentEarth*SIN(DTOR(GeoMeanAnomSun))+;
      4*EccentEarth*VarY*SIN(DTOR(GeoMeanAnomSun))*COS(2*DTOR(GeoMeanLongSun))-0.5*VarY*VarY*;
      SIN(4*DTOR(GeoMeanLongSun))-1.25*EccentEarth*EccentEarth*SIN(2*DTOR(GeoMeanAnomSun)))
      //W HA Sunrise (deg)
      HAsunrise=RTOD(ACOS(COS(DTOR(90.833))/(COS(DTOR(this.lat))*COS(DTOR(sunDecl)))- ;
					 TAN(DTOR(this.lat))*TAN(DTOR(sunDecl))))
      //X Solar Noon (LST)
      solarNoon=(720-4*this.long-eqTime+this.offset)/1440
      sr = (solarNoon-HAsunrise*4/1440)*24*60*60
		ss = (solarNoon+HAsunrise*4/1440)*24*60*60
		//? sr,ss
      this.sunrise=transform(int(sr/3600),"@L 99")+":"+transform(int(sr%3600/60),"@L 99")
      this.sunset =transform(int(ss/3600),"@L 99")+":"+transform(int(ss%3600/60),"@L 99")
	return

	// This function returns the distance in km or miles betweeen two sets of coordinates
	// bMetric = true for km, False for miles
	
	// Programmer: Gaetano De Luisi

	Function CoordsToDistance(lat1,lat2,long1,long2,bMetric)
		this.error.clear()
		try
			if pcount()<4
				this.error.message = "To few arguments (at least 2 sets of coordinates)"
				throw this.error
			elseif pcount()=4 //bMetric not supplied, set default to metric
				bMetric=true
			elseif pcount()>5
				this.error.message = "Too many parameters suplied (max 5)"
				throw this.error
			endif
			if  (lat1>90 or lat1<-90) or (lat2>90 or lat2<-90)
				this.error.message = "A latitude is out of range"
				throw this.error
			endif
			if  (long1>180 or long1<-180) or (long2>180 or long2<-180)
				this.error.message = "A longitude is out of range"
				throw this.error
			endif
			nRet = round((ACOS( SIN(lat1*PI()/180)*SIN(lat2*PI()/180) + COS(lat1*PI()/180)* ;
  					COS(lat2*PI()/180)*COS(long2*PI()/180-long1*PI()/180) ) * 6371000)/ ;
					IIF(bmetric=true,1000,1609.3435),2)
		catch(exception e)			
			this.error.code = 1
			return 0
		endtry
	return nRet

EndClass

Class GTException of Exception
	this.clear()
	function clear	
		this.code = 0
		this.message = "No error"
	return
EndClass
