/*
          Filename: getAdir.cc
            Author: Gary White

Purpose: 
   Supplement to the built-in Visual dBASE 7 getDirectory()
   function.  Adds the ability to specify the text to appear 
   in the title bar of the dialog, a file list (display only) 
   and the ability to create a new directory.

Sep 14, 1998: Original version: 
Sep 17, 1998: Added code to re-read removable drives and
              skipped reading volume from floppies in the
              form onOpen event.
Sep 19, 1998: Added status bar to show free space and an 
              icon for network drives.  Also set the tabstop
              property of the file list to false.
              
April 2021: Images and readme file moved to a folder in the
   dUFLP folder. Minor changes below. -- KJM

Implementation notes:
   This file should have come in a ZIP file which also includes
   seven icons and one bmp image.  If you did not recieve it that 
   way, you may get the complete file at:

            http://www.apptools.com/dbase

   Place all 9 files in your Visual dBASE install /Custom 
   directory. If you choose to place these files in a different
   location, you will need to edit this file and replace the 
   private variable "ccDir" located at the beginning of the 
   function "getADir" with the path where the files are
   placed.

Usage:
  //set procedure to &_dbwinhome.Custom/getADir.cc additive
  set procedure to :dUFLP:getADir.cc
  cMyDir = getADir( cText, lAllowNew)

  eg:  cMyDir = getADir( "Where should it go", true )

     cText - the character string to appear in the title 
             bar of the dialog.

     lAllowNew - logical to specify if you want to allow 
             the user to create a new directory.

Feedback/Problems/Questions:
   I suppose, since this is the original version, you should 
   consider this to be a beta version.  I'd appreciate any 
   feedback, or suggestions.

   If you have problems, or questions with this function, please 
   report them via the Borland news groups, or via the feedback 
   form at http://www.apptools.com/dbase

License:
   This object is FREEWARE. You are free to include this
   object in your own Visual dBASE programs and distribute it
   freely as part of your programs as object code.

   You may freely distribute this object to other developers
   for inclusion in their programs.

   You may NOT sell this object or use it in any way whereby
   it is NOT included in a program.

   You may make any revisions desired to this code.

   Have fun with it!

      Gary

******************************************************************
*                                                                *
*   Warning: This code is not warranteed in any way whatsoever.  *
*   Use it at your own risk. Your use of this code releases Gary *
*   White from any liability whatsoever as regards the use,      *
*   suitability or quality of this object.                       *
*                                                                *
******************************************************************

*/


function getADir
   parameters cText, lAllowNew   // use params to make them private
   local f, retVal
   private ccDir

   retVal = ""                   // initialize the return value

   //ccDir = _dbwinhome + "Custom" // path where this cc and icons are
                                 // stored.  if you place them in a
                                 // location other than your VdB 
                                 // Custom directory change this to
                                 // the correct location.
   ccDir = ":dUFLP:GetADir" // dUFLP source code alias, with folder containing
                            // images used.
   
   do case

      case argCount() == 0       // no params
         cText = ""              // the default is already "Choose a directory
         lAllowNew = true        // default to allowing new directory creation

      case argCount() == 1       // if we only got one param
         if type( "cText" ) == "L"  // and it's logical
            lAllowNew = cText    // move it over
            cText = ""           // and use default text
         else
            lAllowNew = true     // otherwise default to allowing new
         endif

   endcase

   // now, let's check the parameters
   if type( "cText" ) # "C" or type( "lAllowNew" ) # "L"
      showErr( "Parameter type mismatch" )
      return null
   endif

   f = new getDirForm()    // instantiate the dialog
   f.retVal = ""           // initialize form property for 
                           // the return value
   f.mdi := false          // ensure not mdi

   // set the title bar text
   f.text = iif( empty( cText ), "Choose a directory", cText )

   if not lAllowNew        // if not allowing a new directory
      f.newDirButton.visible := false  // hide "New" button
      f.selectedDir.left := 0.3        // move the current selection
                                       // display left
   endif

   f.readModal()              // dialogs are modal <g>

   if not empty( f.retVal )   // user made a selection
      // the return value comes back with a trailing backslash "\"
      // here, we'll trim it off to be consistant with the built-in
      // function.
      retVal := left( f.retVal, len( trim( f.retVal ) ) - 1 )
   else                    // the user chose cancel
      retVal = ""          // return an empty string
   endif

   release f               // release the form

return retVal              // and send back the result
** END HEADER -- do not remove this line
//
// Generated on 09/18/1998
//
parameter bModal
local f
f = new getDirForm()
if (bModal)
   f.mdi = false // ensure not MDI
   f.readModal()
else
   f.open()
endif

class getDirForm of FORM
   with (this)
      onOpen = class::ONOPEN
      scaleFontBold = false
      height = 14.7273
      left = 1.2857
      top = 0.2273
      width = 60.1429
      text = ""
      topMost = true
      smallTitle = true
   endwith


   this.TEXT1 = new TEXT(this)
   with (this.TEXT1)
      height = 0.7273
      left = 0.3
      top = 1.4545
      width = 12
      fontSize = 8
      text = "Directories:"
   endwith


   this.DIRTREE = new TREEVIEW(this)
   with (this.DIRTREE)
      canExpand = class::DIRTREE_CANEXPAND
      onChange = class::DIRTREE_ONCHANGE
      height = 9.4545
      left = 0
      top = 2.1818
      width = 31
      fontSize = 8
      allowEditLabels = false
      allowEditTree = false
      indent = 19
      trackSelect = false
      imageSize = 16
      image = "filename &ccDir.\File.ico"
      selectedImage = "filename &ccDir.\Open.ico"
   endwith


   this.SELECTEDDIR = new ENTRYFIELD(this)
   with (this.SELECTEDDIR)
      when = {;return false}
      height = 1
      left = 5
      top = 0.3
      width = 55
      colorNormal = "BtnText/BtnFace"
      border = false
      fontSize = 8
      value = ""
      borderStyle = 3	// None
   endwith


   this.FILEDISPLAY = new LISTBOX(this)
   with (this.FILEDISPLAY)
      height = 9.4545
      left = 31
      top = 2.1818
      width = 29
      id = 103
      fontSize = 8
      tabStop = false
      colorNormal = "n+/w+"
      colorHighLight = "n+/w+"
   endwith


   this.STATUSBAR = new TEXT(this)
   with (this.STATUSBAR)
      height = 1
      left = 0
      top = 13.7273
      width = 60.1429
      border = true
      anchor = 1	// Bottom
      marginHorizontal = 1
      alignVertical = 1	// Middle
      alignHorizontal = 2	// Right
      fontName = "MS Sans Serif"
      fontSize = 8
      text = ""
      borderStyle = 2	// Lowered
   endwith


   this.NEWDIRBUTTON = new PUSHBUTTON(this)
   with (this.NEWDIRBUTTON)
      onClick = class::NEWDIRBUTTON_ONCLICK
      height = 1
      left = 0.3
      top = 0.15
      width = 3.4286
      text = ""
      speedBar = true
      upBitmap = "filename &ccDir.\newdir.bmp"
      speedTip = "Create New Directory"
      value = false
   endwith


   this.OKBUTTON = new PUSHBUTTON(this)
   with (this.OKBUTTON)
      onClick = {;form.retVal = form.selectedDir.value;form.close()}
      height = 1.1364
      left = 14.5714
      top = 12.1818
      width = 12
      text = "Ok"
      default = true
      upBitmap = "RESOURCE #20"
      value = false
   endwith


   this.CANCELBUTTON = new PUSHBUTTON(this)
   with (this.CANCELBUTTON)
      onClick = {;form.retVal = "";form.close()}
      height = 1.15
      left = 32.2857
      top = 12.1818
      width = 12
      text = "Cancel"
      upBitmap = "RESOURCE #28"
      value = false
   endwith


   this.TEXT2 = new TEXT(this)
   with (this.TEXT2)
      height = 0.7273
      left = 31.5
      top = 1.4545
      width = 12
      fontSize = 8
      text = "Files:"
   endwith


   function onOpen
      local oItem, oSubDir, cDrv

      cDrv = ""

      // create an array to hold references to our icons
      this.aIcons = new array()
      this.aIcons.add("filename &ccDir.\error.ico")   // no root dir
      this.aIcons.add("filename &ccDir.\floppy.ico")  // removable
      this.aIcons.add("filename &ccDir.\drive.ico")   // fixed
      this.aIcons.add("filename &ccDir.\network.ico")   // remote
      this.aIcons.add("filename &ccDir.\cd.ico")      // cdrom
      this.aIcons.add("filename &ccDir.\drive.ico")   // ramdisk

      // create an array of units for displaying disk free space
      form.aUnits = new array()
      form.aUnits.add("Bytes")
      form.aUnits.add("KB")
      form.aUnits.add("MB")
      form.aUnits.add("GB")

      // get the legal drives.  the getDrives function will 
      // populate the top level of our treeview.
      if not form.getDrives( this.dirTree )
         // if we didn't find any...
         showErr( "No drives found" )
         form.close()
         return
      endif
      // get a reference to the first drive
      oItem = this.dirTree.firstChild
      do
         if oItem.nType > 2   // if it has a root directory
            form.getDirectories( oItem )  // populate the top level

            // C:\ root is likely to have a lot of subdirectories, so
            // we'll populate the first level of subdirectories to
            // reduce the time necessary to expand it.  This is a 
            // trade-off that makes the form slower to open, but can
            // greatly increase performance after it's open.
            if right( oItem.text, 4 ) == "(C:)"

               // Save a reference to C:\.  We do this so we can 
               // select it later.  This prevents beating on the 
               // floppy any more than necessary.
               cDrv = oItem

               if oItem.noOfChildren > 0  // if it has children
                  oSubDir = oItem.firstChild    // get the first child
                  do while not empty( oSubDir ) // get its directories
                     form.getDirectories( oSubDir )
                     oSubDir := oSubDir.nextSibling   // loop to next
                  enddo
               endif
            endif
         endif
         oItem := oItem.nextSibling    // get next directory
      until empty( oItem )
      this.dirTree.setFocus()
      if not empty( cDrv )
         cDrv.select()
      endif
      return

   function dirTree_onChange
      local cVolName, cFreespace, nFreespace, nUnits
      // this gets called when the current selection in the 
      // treeview changes

      if type("form.currentDrive") == "U"
         form.currentDrive = chr( 255 )   // initialize
      endif

      // if we've changed drives
      if left( this.selected.fullpath, 1 ) # form.currentDrive
         // get free space
         nFreeSpace = diskspace( asc( left( this.selected.fullpath, 1)) -64 )
         nUnits = 1  // free space is in bytes

         // reduce it to meaningful units of measure
         do while ( nFreeSpace > 1024 ) and ( nUnits < 3 )
            nFreeSpace := nFreeSpace / 1024
            nUnits += 1
         enddo
         if nUnits > 1     // if it's more than Bytes
            // show one decimal place
            form.statusbar.text = str(nFreeSpace, 12, 1) + " " + ;
               form.aUnits[nUnits] + " Disk Free Space "
         else
            // otherwise just show whole bytes
            form.statusbar.text = str(nFreeSpace) + " " + ;
               form.aUnits[nUnits] + " Disk Free Space "
         endif

         // remember which drive we're on
         form.currentDrive := left( this.selected.fullpath, 1 )
      endif

      if ( this.selected.level == 1 )  // if it is a drive item
         // first check to see if the new selection is a removable 
         // drive (i.e. floppy, or cdrom).
         if this.selected.removable
            // if so, get the volume name
            cVolName = form.getVolumeName( this.selected.fullpath )
            if cVolName # this.selected.text    // if it's changed
               // delete children
               do while not empty( this.selected.firstChild )
                  this.selected.firstChild.release()
               enddo

               // if a floppy volume is unnamed, give it a default
               if len( cVolName ) < 6 and this.selected.nType == 2
                  cVolName = "Floppy (" + left( this.selected.fullpath, 2 ) + ")"
               endif

               // assign the new text
               this.selected.text := cVolName

               // re-read the directories
               form.getDirectories( this.selected )
            endif
         endif
      endif

      // get an array of files based on the currently selected path
      form.aFiles = form.fileList( form.dirTree.selected.fullPath )

      // re-assert the dataSource for the file list display
      form.fileDisplay.dataSource = "array form.aFiles"
      // and make sure it goes to the top
      form.fileDisplay.curSel = 1

      // store a reference to the currently selected directory
      form.selectedDir.value = this.selected.fullpath

      return true

   function dirTree_canExpand
      local oItem, oChild

      oItem = this.selected
      if empty( oItem )    // nothing to expand
         return false
      endif

      // If we expand an item, we'll need to find out if the children
      // have children so that the + sign will appear beside them.
      if oItem.noOfChildren > 0     // if the item has children
         oChild = oItem.firstChild  // get a reference to the first child
         if empty( oChild.firstChild ) // if it doesn't already have children
            do
               form.getDirectories( oChild ) // get its sub-directories
               oChild := oChild.nextSibling  // loop to next
            until empty( oChild )            // until we finish all of them
         endif
      endif
      return true

   function fileList( cPath )
      // Returns an array containing a sorted list of filenames.
      local aDir, nCount
      aDir = new array()
      nCount = aDir.dir( cPath + "*.*" )  // get the files
      if nCount > 0                       // if we found any
         aDir.resize( nCount, 1, 1 )      // strip out everything except
                                          // the file names.
      endif
      aDir.sort()                         // sort them
      return aDir

   function getDirectories
      parameter oItem   // needs to be private for TYPE() testing

      // populates the passed treeItem with its subdirectories
      local aDir, nDirCount, nElement, cName

      if empty( oItem ) // needs a treeItem
         return
      endif

      if type( "oItem.firstChild" ) # "U"
         // This one has already been done.
         // However, if it is a removable top level drive, we need
         // to recheck in case the disk has been changed.
         if ( ( oItem.level == 1 ) and ( not oItem.removable ) ) or ;
            ( oItem.level > 1 )
            return
         endif
      endif

      aDir = new array()         // get an array including directories
      nDirCount = aDir.dir( oItem.fullPath + "*.*", "D" )

      // add child directories to the tree
      for nElement = 1 to nDirCount
         if right( aDir[nElement, 5], 1 ) == "D"   // if it's a directory
            if not (aDir[ nElement, 1 ] == "." ;   // ignore "."
               or aDir[ nElement, 1 ] == ".." )    // and ".."
               cName = oItem.fullPath + aDir[ nElement, 1 ]
               oNewItem = new treeItem( oItem, cName )   // add it to the tree
               oNewItem.text = aDir[ nElement, 1 ]       // set the text
               oNewItem.fullPath = cName + "\"           // give it a full 
                                                         // path reference
            endif
         endif
      endfor
      if oItem.noOfChildren > 0  // if we've got sub-directories
         oItem.sortChildren()    // sort them
      endif
      return

   function getDrives( oTree )
      // For this to work, the drives must be mapped to a letter.
      // It won't pick up a UNC reference without the drive mapping.
      local i, oDriveItem, lTemp, nType
      private cDrive

      if type( "GetDriveType" ) # "FP"
         extern CINT GetDriveType( CSTRING ) kernel32 ;
            from "GetDriveTypeA"
      endif

      for i = asc("A") to asc("Z")  // loop through letters A - Z
         // The GetDriveType API call seems to want a literal string
         cDrive = chr( i ) + ":\"   // so we'll build it here
         nType = GetDriveType( "&cDrive." )  // check to see if it exists

         if nType > 1   //  0=Unknown, 1=No root directory
            // create the treeItem
            oDriveItem = new treeItem( oTree, left(cDrive,2) + " Drive")
            oDriveItem.fullPath = cDrive  // store a reference to the path
            oDriveItem.nType = nType      // and save a reference to type
            oDriveItem.removable = ( nType == 2 ) or ( nType == 5 )
            if nType > 2                  // if it's not a floppy
               cVolume = form.getVolumeName( cDrive )
            else                          // otherwise give it a default
               cVolume = "Floppy (" + left( cDrive, 2 ) + ")"
               // we'll add a dummy child here so the + sign will appear
               oDummy = new treeItem( oDriveItem, "Dummy" )
            endif
            oDriveItem.text = cVolume
            oDriveItem.imageSize = 16

            // now assign an image based on the type of drive
            oDriveItem.image = form.aIcons[nType]
            oDriveItem.selectedImage = form.aIcons[nType]
         endif
      endfor
      return not ( empty( oTree.firstChild ) )

   function getVolumeName( cDrive )
      local cVolume, nVolSize, cSysName, nSysNameSize

      if type("GetVolumeInformation") # "FP"
         extern CLOGICAL GetVolumeInformation( ;
            CSTRING, CSTRING, CULONG, CPTR CULONG, ;
            CPTR CULONG, CPTR CULONG, CSTRING, CULONG ;
         ) kernel32 from "GetVolumeInformationA"
      endif
      cVolume=space(128)
      nVolSize=128
      cSysName=space(128)
      nSysNameSize=128
      // get the volume info
      GetVolumeInformation( cDrive, cVolume, nVolSize, 0, 0, 0, cSysName, 128 )

      // clean and format it a little
      cVolume := proper( trim( cVolume ) )

      // build our treeItem text string
      cVolume := left( cVolume, len( cVolume ) - 1 ) ;
         + " (" + left( cDrive, 2 ) + ")"
      return cVolume

   function newDirButton_onClick
      local oNewDir, cDirName, lSuccess, oCurrent
      private cDirectory

      // save a reference to the currently selected treeItem
      oCurrent = form.dirTree.selected

      // get a name for the new directory
      cDirName = form.getNewDirName( oCurrent.fullPath )

      if not empty( cDirName )   // if we got a name
         cDirectory = oCurrent.fullPath + cDirName
         lSuccess = true
         try
            md "&cDirectory"     // attempt to create it
         catch ( exception e )
            lSuccess := false    // if it failed, show the user
            showErr( "Unable to create directory:" + chr(13) + e.message )
         endtry
         if lSuccess             // if it worked
            oNewDir = new TreeItem( oCurrent, cDirName ) // add it to the tree
            oNewDir.Text := cDirName
            oNewDir.fullPath = oCurrent.fullPath + cDirName + "\"
         endif
      endif
      return

   function getNewDirName( cPath)
      // allows creation of a new directory
      local f, cNewDir
      f = new newdirForm() // class definition is later in this file
      f.mdi = false // ensure not MDI
      f.newDirName = ""
      f.pathName.value = cPath
      form.topmost := false   // make this form NOT topmost
      f.readModal()
      cNewDir = f.newDirName  // get the new dir name
      f.release()
      form.topmost := true    // make this form topmost
      return cNewDir
endclass
class newdirForm of form
   // simple form to get a new directory name
   with (this)
      scaleFontBold = false
      height = 7.2273
      left = 18.8571
      top = 2.8636
      width = 35
      text = "New Directory"
   endwith


   this.newDir = new entryfield(this)
   with (this.newDir)
      height = 0.8
      left = 1
      top = 2.3636
      width = 33
      fontSize = 8
      value = ""
   endwith


   this.rectangle1 = new rectangle(this)
   with (this.rectangle1)
      left = 2
      top = 4.1818
      width = 31
      height = 2.3636
      text = "Rectangle1"
      border = false
      fontSize = 8
      borderStyle = 2   // Lowered
   endwith


   this.text1 = new text(this)
   with (this.text1)
      height = 0.7273
      left = 1
      top = 0.1818
      width = 33
      fontSize = 8
      text = "Enter New Directory:"
   endwith


   this.pathName = new entryfield(this)
   with (this.pathName)
      when = {;return false}
      height = 0.8
      left = 1
      top = 1.0909
      width = 33
      colorNormal = "BtnText/BtnFace"
      fontSize = 8
      value = ""
      borderStyle = 10  // Etched Out
   endwith


   this.okButton = new pushbutton(this)
   with (this.okButton)
      onClick = class::okButton_onClick
      height = 1.0909
      left = 4
      top = 4.8182
      width = 12
      text = "Ok"
      default = true
      value = false
   endwith


   this.cancelButton = new pushbutton(this)
   with (this.cancelButton)
      onClick = {;form.close()}
      height = 1.0909
      left = 19
      top = 4.8182
      width = 12
      text = "Cancel"
      topmost = true
      value = false
   endwith


   function okButton_onClick
      form.newDirName = form.newDir.value
      form.close()
      return
endclass

function showErr( cMessage )
   msgbox( cMessage, "Error", 16 )
   return
