#define FileVersion 2.0
/*
    SeqValue.cc
    Programmer: Ken Mayer
    Date......: March 3, 2005
    Notes.....: This is based on the concepts in the old XDML
                SEQValue.cc class that Romain (Strieff) created 
                ages past.

                The concept is to provide an autoincrement
                field for those who are leary of the one
                built in to the dBF table format, or are using
                a SQL database that does not have this capability, 
                or -- if you need to have multiple autoincrement
                type fields in a row for some reason or other
                (I am not sure I can think of a reason why,
                but one never knows) you could do that using
                this code ... (the .dBF table format allows
                one autoincrement field per table).

                This version should (hopefully) do what the old
                one did, but it works differently. The biggest
                difference is that this is not a User Interface
                based class -- it can be used in code, it
                can be used on a form, whatever you need.

                It requires the name of the table (not the
                rowset object), the name of the field, and
                then it will attempt to perform the autoincrement.

                If these values are not in the table that it
                uses (and it will create if needed), then it
                will create the row, and default the next value
                to 1.

                *** WARNING:
                You may have difficulties using this with
                a server such as MSSQL, as that database
                requires a prefix to a table: "dbo.TableName"
                when creating/using a table. Not sure about
                all databases. If that's the case, you might
                want to modify this to deal with it.

    Updates:    September, 2005 -- updated to use a database
                alias, if provided. Works on a local table
                if no database. 

                Also added Reset method, which resets the 
                value to what you need it to be. See below
                under "usage". 

                And finally, as there is no need for the .MDX
                (no need for the index tag), it has been
                removed. If you are already using this
                it shouldn't cause any problems.

    Usage:      set procedure to :dUFLP:SeqValue.cc additive
                oSeq = new SeqValue( [<database alias>] )
                oSeq.Increment( <tablename>, <fieldname> )
                oSeq.Reset( <tablename>, <fieldname>, <newValue> )
                oSeq.release()

                Example:
                set proc to :dUFLP:SeqValue.cc additive
                oSeq = new SeqValue( "MyAlias" )
                // Note that "MyAlias" is optional, if no database
                // alias is passed to this class, we will use
                // _app.databases[1] -- which is the current folder.
                nValue = oSeq.Increment( "MyTable", "MyField" )
                form.rowset.fields["MyAutoInc"].value := nValue
                oSeq.release()

                // to use this on a form, you could do something
                // with the rowset's canSave or onSave events,
                // and call this, setting the value at that
                // point, OR if you need the value before
                // saving the row, call it in the beginAppend(),
                // for example ... then store the value to
                // the field.  As an example:
                function rowset_beginAppend
                   set proc to :dUFLP:SeqValue.cc additive
                   oSeq = new SeqValue()
                   this.fields["MyAutoInc"].value := ;
                          oSeq.Increment( "MyTable", "MyField" )
                   oSeq.release()
                   close proc :dUFLP:SeqValue.cc
                return

    Methods:    InitTable() // Create table if needed, otherwise
                            // open table used to store/increment
                            // values as a property of the
                            // class  ** PROTECTED **
                release()   // as it sounds, should be called to
                            // release the object and close things down.
                Increment() // performs the increment, requires
                            // you pass the tablename and the fieldname
                            // as parameters.
                            // Example:
                            //    oSeq.Increment( "MyTable", "MyField" )
                Reset()     // Resets the autoinc value in a field.
                            // Important: to start at 1, you must
                            // reset to zero. 
                            // Example:
                            //    oSeq.Reset( "MyTable", "MyField", 0 )

    Side Effects: This creates the SeqValue in the current
                  folder that the code is being executed in
                  by default. If you use the optional
                  database alias when instantiating this
                  then the code should create the SeqValue
                  table in the folder specified by the
                  alias.
*/
class SeqValue
   protect InitTable
   if pCount() == 0
      this.DataBase = ""
   else
      this.DataBase = ArgVector( 1 ) // we have a database alias
   endif

   // startup code:
   if not class::InitTable()
      cancel // stop processing
   endif

   function InitTable
      local oIndex, cDataBase
      private cCmd

      // Create the table if it does not (yet) exist:
      cDatabase = this.database
      this.lDBCreated = false
      // if we have no database, we assume the
      // working directory/default database object:
      if empty( cDatabase )
         this.oDB = _app.databases[1]
      else
         // otherwise, we create a database object
         // using the database alias:
         try
            this.oDB = new database()
            this.oDB.databaseName := cDatabase
            this.oDB.active       := true
            this.lDBCreated       := true
         catch( Exception E )
            msgbox( "Error: "+e.code+" - "+e.message+chr(13)+;
                    "Line: "+e.LineNo,;
                    "Error setting up database", 16 )
            return false
         endtry
      endif

      // Check to see if the table exists:
      if not this.oDB.tableExists( "SeqValue" )
         // create table
         cCmd = [create table "SeqValue" (]+;
                [IncTable  character( 32 ),]+;
                [IncField  character( 32 ),]+;
                [IncValue  integer )]
         this.oDB.executeSQL( cCmd )
      endif

      // Create query for table:
      this.SeqTable         = new query()
      this.SeqTable.database := this.oDB
      this.SeqTable.sql    := "select * from SeqValue"
      this.SeqTable.active := true
      // place us at the first row:
      this.SeqTable.rowset.first()

   return true
   // End of Method: InitTable()

   function release
      // release the query object after setting
      // the active property to false
      this.SeqTable.active := false
      release object this.SeqTable
      // if the database object was created:
      if this.lDBCreated
         this.oDB.active := false
         release object this.oDB
      endif
      // and release this object, and we're done.
      release object this
   return null

   function Increment( cTable, cField )
      // here's where we actually deal with incrementing
      // the value ...

      // error checking for required parameters
      if pCount() < 1
         msgbox( "You must pass a tablename to be used"+;
                 "for the increment process.",;
                 "SeqValue Error", 16 )
         return
      endif
      if pCount() < 2
         msgbox( "You must pass a fieldname to be used"+;
                 "for the increment process.",;
                 "SeqValue Error", 16 )
         return
      endif

      // moving on, we need to see if the table and field
      // names are in the table:
      local bAddRow
      bAddRow = false
      this.SeqTable.rowset.filterOptions := 2 // ignore case
      this.SeqTable.rowset.filter := [IncTable = ']+cTable+[']
      if not this.SeqTable.rowset.endOfSet
         // we're not at the end of the rowset, so
         // look for the fieldname:
         this.SeqTable.rowset.locateOptions := 2 // ignore case
         if this.SeqTable.rowset.applyLocate( [IncField = ']+cField+['] )
            // we found it!
            this.SeqTable.rowset.fields["IncValue"].value++
            this.SeqTable.rowset.save()
            return this.SeqTable.rowset.fields["IncValue"].value
         else
            // the field doesn't exist, but the table is
            // referenced, so we need to still add a row:
            bAddRow := true
         endif
      else
         // The table is not referenced, so we need to add a row:
         bAddRow := true
      endif

      // if we need to add the row:
      if bAddRow
         this.SeqTable.rowset.beginAppend()
         this.SeqTable.rowset.fields["IncTable"].value := cTable
         this.SeqTable.rowset.fields["IncField"].value := cField
         this.SeqTable.rowset.fields["IncValue"].value := 1
         this.SeqTable.rowset.save()
         return 1 // we created a new row, starting at 1
      endif
      // there shouldn't need to be an "else", as at this point,
      // we should have trapped everything, and be set ...
   return "Error" // just for completeness' sake
   // End of Method: Increment()

   function Reset( cTable, cField, nNewVal )
      // Reset the value to 'nNewVal'

      // error checking for required parameters
      if pCount() < 1
         msgbox( "You must pass a tablename to be used "+;
                 "for the increment process.",;
                 "SeqValue Error", 16 )
         return "Error"
      endif
      if pCount() < 2
         msgbox( "You must pass a fieldname to be reset ...",;
                 "SeqValue Error", 16 )
         return "Error"
      endif
      if pCount() < 3
         msgbox( "You must pass a new value to be used "+;
                 "for the increment field.",;
                 "SeqValue Error", 16 )
         return "Error"
      endif

      private n
      n = nNewVal
      if type( "n" ) <> "N"
         msgbox( "Value to be used must be numeric "+;
                 "(for the increment field).",;
                 "SeqValue Error", 16 )
         return "Error"
      endif

      if nNewVal < 0
         msgbox( "Value to be used must be 0 or greater "+;
                 "(for the increment field).",;
                 "SeqValue Error", 16 )
         return "Error"
      endif
         
      // moving on, we need to see if the table and field
      // names are in the table:
      local bRowFound
      bRowFound = false
      this.SeqTable.rowset.filterOptions := 2 // Ignore case
      this.SeqTable.rowset.filter := [IncTable = ']+cTable+[']
      if not this.SeqTable.rowset.endOfSet
         // we're not at the end of the rowset, so
         // look for the fieldname:
         this.SeqTable.rowset.locateOptions := 2 // Ignore case
         if this.SeqTable.rowset.applyLocate( [IncField = ']+cField+['] )
            // we found it!
            bRowFound := true
            this.SeqTable.rowset.fields["IncValue"].value := nNewVal
            this.SeqTable.rowset.save()
            return this.SeqTable.rowset.fields["IncValue"].value
         else
            // We didn't find it ... 
            bRowFound := false
         endif
      else
         // We didn't find it ...
         bRowFound := false
      endif

      // if we need to add the row:
      if not bRowFound
         msgbox( "Did not find table '"+cTable+"' with field '"+cField+"'."+chr(13)+;
                 "Cannot reset field value.",;
                 "SeqValue Error", 16 )
         return "Error!"
      endif
      // there shouldn't need to be an "else", as at this point,
      // we should have trapped everything, and be set ...
   return "Error" // just for completeness' sake
   // End of Method: Increment()

endclass

