// Metric: 0 - Chars only
/*
===================================================================
   Program: StatusBar.cc
    Author: Gary White (garyw@qni.com)
   History: June 19, 1998 - Original version
   			May 29, 1999 - fixed bug that raised an exception if
            a control had no statusMessage property and change to
            using a dummy toolbar's onUpdate instead of a timer
            to refresh the display.

   Description: 
   	This is a status bar that can be used on SDI forms.  It includes 
   	panels which display the statusMessage of controls, the status 
      of the Ins key, the status of CapsLock key, and the form's 
      rowset state.

   Implementation:
   	From the command window, type: 

	    	set proc to :dUFLP:StatusBar.cc additive

      Then open your form in the form designer.  The status bar will
      appear on the "Custom" page of the Component palette.  Just 
      drag it onto your form.

      Alternative:
      	set procedure to statusbar.cc additive
      	f = new form()
         f.sb = new statusbar(f)
         f.open()

	Notes:
   	The status bar uses a timer object firing once per second to
      update the status of the CapsLock and Ins keys and the state
      of the rowset.  You may see a short delay (normally less than
      one second) between your actions and the resulting change in 
      the display.

      If the form does not have a rowset, the right panel will 
      remain blank. This is checked in the refresh method called by
      the timer, so it will pick up a rowset defined on the fly 
      after the status bar has been initialized.

      The individual panels of the status bar are implemented using
      entryfields wrapped in a container object, so resource usage 
      is minimized. 

      The status bar is based on a container which is anchored to 
      the bottom of the form.  If you want to use a notebook, or 
      container on your form and have it anchored to the bottom, 
      center, or container, you should install the status bar first.
      Then, installing the notebook, or container, will place them
      above the status bar.

      If you create a control on the fly, it will not automatically
      display its status message.  If you want it to do so, you'll 
      need to set this up manually as in the following example:

e = new entryfield( form )
e.onGotFocus = {;form.StatBar.StatusMsgText.value = this.statusMessage}
e.statusMessage = "Hello"

		This is only required for controls created after the form has
      opened. When the status bar initializes itself, it creates a 
      property of the form, called "StatBar" so it can reference 
      itself regardless of what you name it.  Thus, form.StatBar 
      will always refer to the status bar.

   Enjoy,
   Gary
===================================================================
*/

class STATUSBAR(parentObj, name) of CONTAINER(parentObj, name) custom
   with (this)
      left = 0
      top = 7
      width = 53
      height = 1
      metric = 0			// Chars
      borderStyle = 1	// Raised
      anchor = 1			// Bottom
      pageno = 0			// Show on all pages
      onDesignOpen = class::Init
   endwith

   this.StatusMsgText = new ENTRYFIELD(this)
   with (this.StatusMsgText)
      left = 1
      top = .1
      width = 32.1
      height = .75
      colorNormal = "btntext/btnface"
      border = true
      fontSize = 8
      value = ""
      pageno = 0
      borderStyle = 2			// lowered
      when = {;return false}	// don't get focus
      mousePointer = 1			// keep the arror pointer
   endwith

   this.InsertStateText = new ENTRYFIELD(this)
   with (this.InsertStateText)
      left = 29.3
      top = .1
      width = 3.2
      height = .75
      colorNormal = "btntext/btnface"
      border = true
      fontSize = 8
      value = ""
      pageno = 0
      borderStyle = 2			// lowered
      when = {;return false}	// don't get focus
      mousePointer = 1			// keep the arror pointer
   endwith

   this.CapsLockStateText = new ENTRYFIELD(this)
   with (this.CapsLockStateText)
      left = 34.75
      top = .1
      width = 5
      height = .75
      colorNormal = "btntext/btnface"
      border = true
      fontSize = 8
      value = ""
      pageno = 0
      borderStyle = 2			// lowered
      when = {;return false}	// don't get focus
      mousePointer = 1			// keep the arror pointer
   endwith

   this.RowsetStateText = new ENTRYFIELD(this)
   with (this.RowsetStateText)
      left = 40
      top = .1
      width = 10.5
      height = .75
      colorNormal = "btntext/btnface"
      border = true
      fontSize = 8
      value = ""
      pageno = 0
      borderStyle = 2			// lowered
      when = {;return false}	// don't get focus
      mousePointer = 1			// keep the arror pointer
   endwith

///////////////////// Methods follow /////////////////////

	function Init
   	// Check to see if we're attached to a form
      if type("this.parent.autoCenter") # "L"
	      // if not, inform the user
      	msgbox("Status bar requires a form as the parent",;
         	"Status bar error", 16)
         return // and exit
      endif
      this.AdjustSize()  // adjust the size
		return

   function onOpen
      this.init()						// just checking

		// Hang a reference to the status bar on the form
      form.StatBar = this


      // create a dummy toolbar so we can use its
      // onUpdate routine to refresh our display
      this.t=new toolbar()
      // hide it off screen
      this.t.floating := true
      this.t.top := -300
		// and attach it to our form
      this.t.attach( this.parent )
      this.t.parent = this
      this.t.onUpdate = {;this.parent.refresh()}

      // resize the status bar when the form is resized
      // if the form has no onSize event already
      if empty( form.onSize )
      	// just have it call our AdjustSize routine
      	form.onSize = {;this.StatBar.AdjustSize()}
      else		// otherwise
      	// save the original routine
      	form.sOnSize = form.onSize
         // and call it and then our AdjustSize routine
         form.onSize = {;this.sOnSize(); this.StatBar.AdjustSize()}
      endif

      // finally install the display of the statusMessages
      this.InstallStatDisplay( form )
		return

   function InstallStatDisplay
   // uses objects onGotFocus routines to display their statusMessages
   	parameters oObject // use parameters statement to make oObject private
   	private oControl	// again private because type() needs it that way
      oControl = oObject.first	// get the first object
      do
      	try
         	// if the control does not have a statusMessage 
            // property create one to avoid errors
         	if type('oControl.statusMessage')=='U'
            	oControl.statusMessage = ''
            endif
            if empty( oControl.onGotFocus )	// no existing onGotFocus
            	// install ours
               oControl.onGotFocus = {;form.StatBar.StatusMsgText.value = this.statusMessage}
            else	// otherwise
            	// save the original
               oControl.sOnGotFocus = oControl.onGotFocus
               // install ours and then call the original
               oControl.onGotFocus = {;form.StatBar.StatusMsgText.value = this.statusMessage; this.sOnGotFocus()}
            endif
         catch (exception e)
         // Ignore errors assuming that the error is because 
         // the control does not have an onGotFocus.
         endtry

         // now check to see if this is a container or notebook
      	if type( "oControl.first" ) # "U"
         	// if so, recurse the call to install in its objects
         	this.InstallStatDisplay( oControl )
         endif
         oControl := oControl.before
      until oControl.name = oObject.first.name
      return
   	
   function Refresh
   	local n
   	// check form's rowset to determine text for panel 2
      if type("form.Rowset") == "U"
      	// if no rowset, just blank the panel
      	this.RowsetStateText.value := ""
      else
         do case
            case form.rowset.state = 0 // closed
               this.RowsetStateText.value := "Closed"
            case form.rowset.state = 1 // browse
               this.RowsetStateText.value := "View Record"
            case form.rowset.state = 2 // editing
               this.RowsetStateText.value := "Editing"
            case form.rowset.state = 3 // appending
               this.RowsetStateText.value := "New Record"
            case form.rowset.state = 4 // filter
               this.RowsetStateText.value := "Filter"
            case form.rowset.state = 5 // locate
               this.RowsetStateText.value := "Locate"
            otherwise
               this.RowsetStateText.value := "Unknown"
         endcase
      endif

      // extern to check the status of CapsLock and Ins keys
      if type( "GetKeyState" ) # "FP"
         extern CSHORT GetKeyState(CINT) user32
      endif
      this.CapsLockStateText.value := ;
         iif( GetKeyState( 0x14 ) == 0, "caps","CAPS" )
      this.InsertStateText.value := ;
         iif( GetKeyState( 0x2D ) == 0, "Ins", "Ovr" )
		return

   function AdjustSize
      this.left = -1
      this.width = form.width + 2
      this.StatusMsgText.width = this.width - 21.9
      this.InsertStateText.left = this.width - 20.5
      this.CapsLockStateText.left = this.width - 17.25
      this.RowsetStateText.left = this.width - 12
   return

endclass
