/*

   FileName:      SysInfo.cc
   Author:        Gary White

   Description:   Generic object that surfaces a number of system
                  descriptive properties.  Primarily designed for
                  use in populating a Help->About box, or may be
                  called to display a generic about box with
                  system information.

   =============================================================================
   Version History:
   =============================================================================

   June 15, 1999  Original version

   June 21, 1999  Moved lines setting procedures to the init method
                  to ensure that they would be executed.

                  Changed the ShowAbout method to center the about
                  box on the form that called it.

   June 23, 1999  Modified the centering routine to avoid the about
                  box being partially off-screen.

   July 1, 1999   Modified the siGetDBEngine routine based on code
                  supplied by Vic McClung that corrects a problem
                  where multiple paths are shown in the registry.
                  Change to use GetVersionEx and return additional
                  Windows version info including the NT Service Pack
                  and the Windows build number.

   Jan 15, 2000   Fixed the operating system build number and cleaned
                  up the formatting of the operating system string.

   Feb 29, 2000   Minor formatting cleanup.

   Mar 21, 2011   Andrew Shimmin

                  -  set version to v1.3
                  -  Added support for dBPLUS v2.70
                  -  Added methods:

                        getDbaseAllUsersHomePath()
                        getDbaseBuildNumber()

                  -  Modified structure class from:

                        structure.prg/strucmem.dll

                        to:

                        ccs_structures.cc

                  -  Modified registry class from:

                        registry.prg

                        to:

                        rmRegistry.cc

                  -  Modified method siGetMemoryInfo() to use:

                        GlobalMemoryStatusEx() windows API

                        instead of:

                        GlobalMemoryStatus()

                        The use of the GlobalMemoryStatusEx() windows API will
                        provide more accurate memory usage numbers than its
                        predecessor.

   Dec 13, 2011   Andrew Shimmin

                  -  set version to v1.4

                  -  Added methods:

                     is_OS_64Bit()
                     is_Process_Elevated()
                     is_OS_W2K_or_later()

                  -  Modified the memory properties so they had correct values
                     when the class is run on a 64-bit operating system.

   Nov 9, 2012   Andrew Shimmin

                  -  set version to v1.5

                  -  Added methods:

                     get_OS_Version()
                     get_OS_VersionId()
                     get_OS_VersionBuild()

                  -  Added Windows 8 support.

   Nov 10, 2012   Andrew Shimmin

                  -  set version to v1.6

                  -  Fixed version string to support compatability mode setting
                     from Vista/Server 2008 onwards

                     get_OS_version() will return the correct version.
                     is_OS_64bit() will be true for a native 64bit system version.


   =============================================================================
   Dependencies:
   =============================================================================

   SysInfo.h         (included with this cc)
   Windef.h          (included with this cc)
   Winreg.h          (included with this cc)

   rmRegistry.cc     (included with this cc)
   ccs_structures.cc (included with this cc)                                  // MUST be >= v0.0.0.7


   =============================================================================
   Notes:
   =============================================================================

   IDE:

      rmRegistry.cc and ccs_structures.cc must be included in your current folder.

   Runtime:

      rmRegistry.cc and ccs_structures.cc must be included in your executable file.

   =============================================================================
   Usage:
   =============================================================================

   set procedure to sysinfo.cc additive

   local s

   s = new sysInfo()
   with ( s )
      appName = 'My Super Application'
      appVersion = '1.0'
      Copyright = 'Copyright 1999, by Me'
      UserName = 'My Customer'
   endwith

   s.ShowAbout()                                                              // display an about box

   ... on application close ...

   s.release()

   Alternatively, you may instantiate a SysInfo object
   and query its properties to populate your own about
   dialog box.

   =============================================================================
   Properties:
   =============================================================================

   AppName:                   Name of your app. You must set this manually
   AppVersion:                Your app version. You must set this manually
   AppCopyright:              Copyright statement. You must set this manually

   PhysicalMemory:            The amount of actual physical memory.
   AvailableMemory:           The amount of physical memory currently available.
   MemoryLoad                 The approximate percentage of physical memory that
                              is in use.

   ColorDepth:                Number of colors supported by the display
   ComputerName:              The computer name as reported by the API

   DbEngine:                  BDE version and date
   DdeName:                   Application DDE Name
   DefaultTableLevel:         Table level as set in the BDE Admin applet
   LangDriver:                Language driver as set in yourapp.ini
   LocalShare:                Value of Local Share as set in BDE Admin applet

   NumProcessors:             Number of processors (significant only on NT)
   OperatingSys:              Operating System (Win95, 98, or NT) and version
   PageFileSize:              Size of swap file                               // this is out of date an inaccurate
   PageFileUsage:             Percent of swap file usage                      // this is out of date an inaccurate
   Processor:                 Type of processor
   RunTimeEngine:             Version of VdB, or VdB runtime
   ScreenRes:                 Screen resolution in pixels

   l64bit                     Operating system is 64-bit
   lAdministrator             Running with administrative privilege
   lW2KOn                     Operating system is W2K onwards

   =============================================================================
   Events:
   =============================================================================

   None

   =============================================================================
   Methods:
   =============================================================================

   Refresh                    Refreshes information that may change
   Release                    Releases the object from memory
   SetDdeName                 Sets the app's DdeServiceName.
                              The old DdeServiceName is reset in
                              the release() method.
   ShowAbout                  Shows a generic About box

   getDbaseAllUsersHomePath   The dbase user's home path.
   getDbaseBuildNumber        The dBase build number.

   is_OS_64Bit                Returns TRUE if 64-Bit Operating system.
   is_Process_Elevated        Returns TRUE if the calling process is running with
                              full administrative privilege.
   is_OS_W2K_or_later         Returns TRUE if operating system is W2K or later.

   get_OS_Version             Returns the system version string (Windows 7 etc).
   get_OS_VersionId           Returns the system version IDeg (6.1 etc).
   get_OS_Build               Returns the system build number (7600 etc).

*/

class SysInfo of timer

   #include windef.h
   #include winreg.h
   #include sysinfo.h

   protect  init, siGetComputerName, siGetDbEngine, siGetDDEServName, ;
            siGetLangDriver, siGetLocalShare, siGetMemoryInfo, ;
            siGetNumProcessors, siGetOperatingSys, siGetRunTimeEngine, ;
            siGetVideoInfo, siGetTableLevel

   // These first three properties must be set manually by
   // your code. The values inserted here are informational
   // only and are intended to be overwritten with
   // information about your application.

   this.AppName = 'System Information'
   this.AppVersion = 'v 1.6'
   this.AppCopyright = 'Copyright 2000, Gary White'

   // these properties are intended to be set automatically by the class

   this.AvailableMemory = ''
   this.ComputerName = ''
   this.DbEngine = ''
   this.DdeName = this.siGetDDEServName()
   this.LangDriver = ldriver()
   this.LocalShare = ''
   this.NumProcessors = ''
   this.PageFileSize = ''
   this.PageFileUsage = ''
   this.PhysicalMemory = ''
   this.operatingSys = ''
   this.memoryLoad = ''
   this.Processor = ''
   this.RunTimeEngine = version(0)
   this.ScreenRes = ''
   this.ColorDepth = ''

   this.DefaultTableLevel = ''
   this.l64bit =false
   this.lAdministrator = false
   this.lW2KOn = false

   this.lServer = false
   this.lWorkstation = false
   this.lDomainController = false

   this.cOperatingSysVersion = []
   this.nOperatingSysVersionId = 0
   this.nOperatingSysVersionBuild = 0

   this.init()

   function Init
/*
      #include structapi.h
      #include windef.h
      #include winreg.h
      #include sysinfo.h
*/
      #define  VER_NT_DOMAIN_CONTROLLER         0x00000002
      #define  VER_NT_SERVER                    0x00000003
      #define  VER_NT_WORKSTATION               0x00000001

      #define  VER_SUITE_PERSONAL               0x00000200

      #define  VER_SUITE_DATACENTER             0x00000080
      #define  VER_SUITE_ENTERPRISE             0x00000002

      #define  VER_SUITE_TERMINAL               0x00000010                    // Terminal Services is installed
      #define  VER_SUITE_SINGLEUSERTS           0x00000100                    // Remote Desktop is supported

      #define  VER_SUITE_WH_SERVER              0x00008000                    // Windows Home Server is installed.

      #define  SM_TABLETPC                      86                            // Windows XP Tablet PC edition; otherwise, 0
      #define  SM_MEDIACENTER                   87                            // Windows XP, Media Center Edition, 0 if not.
      #define  SM_STARTER                       88                            // Windows XP Starter Edition; otherwise, 0

      local nNumberP, nProcessor, nPlatform, vMinor, vMajor, ;
            cExtra, nBuild, v, s

      local cHomePath,wProductType,lWow64

      cHomePath = []
      wProductType = 0
      lWow64 = false

      // Processor and operating system won't change during the
      // class life, so we'll just grab that information here.

      try

         cHomePath := class::getDbaseAllUsersHomePath()

         set procedure to rmRegistry.cc additive
         set procedure to ccs_structures.cc additive

         if type([GetCurrentProcess]) # [FP]
            extern CHANDLE GetCurrentProcess(CVOID) Kernel32
         endif

         try
            if type([IsWow64Process]) # [FP]
               extern CLOGICAL IsWow64Process(CHANDLE,CPTR CLOGICAL) kernel32
            endif
         catch (exception e)
         endtry

         try
            if IsWow64Process(GetCurrentProcess(),lWow64)                     // check for 64 bit
               this.l64Bit := lWow64
            else
               this.l64Bit := false
            endif
         catch (exception e)
            this.l64Bit := false
         endtry

         // get operating system

         v = new VersionInfo()                                                // create a version info structure

         if GetVersionEx(v)                                                   // fill the structure

            nPlatform = v.dwPlatformId()
            vMajor = int(v.dwMajorVersion())
            vMinor = int(v.dwMinorVersion())
            nBuild = int(bitand(v.dwBuildNumber(),0xffff))
            wProductType := v.wProductType()                                  // get product type mask
            cExtra = trim(v.szCSDVersion())

            this.cOperatingSysVersion := []
            this.nOperatingSysVersionId := val(ltrim(str(vMajor))+[.]+ltrim(str(vMinor)))
            this.nOperatingSysVersionBuild := nBuild

            // get system version
            //
            // Note: The test for a server MUST be first since it is a bit test

            do case
               case  class::bitTest(VER_NT_SERVER,wProductType)               // server
                     this.lServer := true
               case  class::bitTest(VER_NT_WORKSTATION,wProductType)          // workstation
                     this.lWorkstation := true
               case  class::bitTest(VER_NT_DOMAIN_CONTROLLER,wProductType)    // domain controller
                     this.lDomainController := true
            endcase

            do case

               case nPlatform == VER_PLATFORM_WIN32s                          // Win 3.x should not happen
                     this.cOperatingSysVersion = 'Windows'
               case nPlatform == VER_PLATFORM_WIN32_WINDOWS                   // Win95, or Win98, or ?

                     do case
                        case vMajor == 4 and vMinor == 0
                           this.cOperatingSysVersion = 'Windows 95'
                        case vMajor == 4 and vMinor == 10
                           this.cOperatingSysVersion = 'Windows 98/SE'
                        otherwise  // ? unknown
                           this.cOperatingSysVersion = 'Windows'
                     endcase

               case nPlatform == VER_PLATFORM_WIN32_NT                        // WinNT

                     do case
                        case  vMajor == 4
                              this.cOperatingSysVersion = 'Windows NT'
                        case  vMajor == 5.and.vMinor == 0
                              this.cOperatingSysVersion = 'Windows 2000'
                        case  vMajor == 5.and.vMinor == 1
                              this.cOperatingSysVersion = 'Windows XP'
                        case  vMajor == 5.and.vMinor == 2
                              this.cOperatingSysVersion = 'Windows Server 2003'
                        case  vMajor == 6.and.vMinor == 0
                              do case
                                 case  this.lWorkstation
                                       this.cOperatingSysVersion = 'Windows Vista'
                                 case  this.lServer
                                       this.cOperatingSysVersion = 'Windows Server 2008'
                              endcase
                        case  vMajor == 6.and.vMinor == 1
                              do case
                                 case  this.lWorkstation
                                       this.cOperatingSysVersion = 'Windows 7'
                                 case  this.lServer
                                       this.cOperatingSysVersion = 'Windows Server 2008 R2'
                              endcase
                        case  vMajor == 6.and.vMinor == 2
                              do case
                                 case  this.lWorkstation
                                       this.cOperatingSysVersion = 'Windows 8'
                                 case  this.lServer
                                       this.cOperatingSysVersion = 'Windows Server 2012'
                              endcase
                        otherwise
                              this.cOperatingSysVersion = 'Undefined'
                     endcase

                     if not empty( cExtra )
                        this.cOperatingSysVersion += [ ]+cExtra
                     endif

            endcase

            v.release()

         else
             this.cOperatingSysVersion = os()
         endif

      catch (exception e)
         this.cOperatingSysVersion := os()
      endtry

      try

         // get processor info

         s = new SystemInfo()

         GetSystemInfo(s)
/*
         if ( nPlatform == 2 )                                                // NT
            nProcessor = s.wProcessorLevel()
         else
            nProcessor = s.dwProcessorType()
         endif

         do case
            case ( nProcessor == 3 )
               this.processor :=  "386"
            case ( nProcessor == 4 )
               this.processor :=  "486"
            case ( nProcessor == 5 OR nProcessor == 586)
               this.processor :=  "Pentium"
            case ( nProcessor == 6 )
               this.processor :=  "Pentium II or Pro"
            otherwise
               this.processor :=  ltrim( trim( str( nProcessor ) ) )
         endcase

         nNumberP   = s.dwNumberOfProcessors()
         this.numProcessors :=  '' + int(nNumberP)

         if this.numProcessors == 1
            this.processor += ' (single processor)'
         else
            this.processor += ' ('+this.numProcessors+' processors)'
         endif
*/
         this.processor := ltrim(str(s.dwNumberOfProcessors()))

         s.release()

      catch (exception e)
         this.processor := 'Undetermined'
         this.numProcessors := 'Undetermined'
      endtry

      this.ComputerName := this.siGetComputerName()
      this.siGetMemoryInfo()
      this.siGetVideoInfo()
      this.DbEngine = this.siGetDBEngine()
      this.LocalShare = this.siGetLocalShare()
      this.DefaultTableLevel = this.siGetTableLevel()

      /*

         Get administrative privilege setting.

         This setting will be TRUE if the current process is running with full
         administrator privilege. This is the same as:

         A full admin access token.
         Running elevated.


      */

      if vMajor >= 5                                                          // W2K onwards?
         if type([IsUserAnAdmin]) # [FP]
            extern CLOGICAL IsUserAnAdmin(CVOID) shell32 from 680             // notice external ordinal 680!
         endif
         this.lW2kOn := true                                                  // set W2K onwards
         this.lAdministrator := IsUserAnAdmin()                               // set administrator privilege
      endif

      release nNumberP, nProcessor, vMinor, vMajor, v, s

      return

   function Refresh
      this.ComputerName := this.siGetComputerName()
      this.siGetMemoryInfo()
      this.siGetVideoInfo()
      return

   function Release
      _app.ddeServiceName = iif(type('this.savedDdeName')=='C',this.savedDdeName,'VDB')
      release object this
      release this
      return

   function SetDdeName
      parameters cNewName
      local bRet
      bRet = true
      if not empty( cNewName ) and type('cNewName') == 'C'
         this.savedDdeName = _app.ddeServiceName
         _app.ddeServiceName = cNewName
      else
         bRet := false
      endif
      this.ddeName := this.siGetDDEServName()
      return bRet

   function bitTest(nMask,nValue)
   //
   // Method to perform a bit test.
   //
   // Input:
   //
   //    nMask    =  bit mask to use
   //    nValue   =  value to test if the nMask bits are set
   //
   // Output:
   //
   //    true     =  the nMask bits are set in nValue
   //    false    =  the nMask bits are NOT set in nValue
   //

      return (bitAnd(nMask,nValue) = nMask)

   function ShowAbout
      // Shows an About box with system information
      // If called from an MDI application, it will center
      // the about box on _app.framewin.  If called from an
      // SDI application, it will center the about box on
      // the currently active form.  If an error is
      // encountered the form is centered on the screen.

      local f, l, t, r, b, h, w

      try
         this.refresh()
         f=new AboutBoxForm()
         f.sysInfo = this
         f.mdi := false
         try
            if not _app.frameWin.visible
               // we're in an mdi app
               hParent = GetActiveWindow()
            else
               // otherwise find out who called us
               hParent = _app.framewin.hWnd
            endif

            // get position and size of our "parent"

            sWin = new _RECT()

            if GetWindowRect(hParent,sWin)

               l = sWin.left()
               t = sWin.top()
               r = sWin.right()
               b = sWinbottom()
               h = b-t
               w = r-l

               // now center our about form on the parent
               f.left := l + (w/2 - f.width/2)
               // don't set left to less than zero
               f.left := max( f.left, 0 )
               // don't set left so that the right side is off screen
               f.left := min( getSystemMetrics(0)-f.width, f.left)
               f.top := t + (h/2 - f.height/2)
               // don't set top to less than zero
               f.top := max( f.top, 0 )
               // don't set top so that the bottom is off screen
               f.top := min( getSystemMetrics(1)-f.height, f.top)

            else
               f.autoCenter := true
            endif

            sWin.release()
/*
            sWin=new rect()
            if GetWindowRect( hParent, sWin.value)
               l=sWin.getMember('left')
               t=sWin.getMember('top')
               r=sWin.getMember('right')
               b=sWin.getMember('bottom')
               h=b-t
               w=r-l
               // now center our about form on the parent
               f.left := l + (w/2 - f.width/2)
               // don't set left to less than zero
               f.left := max( f.left, 0 )
               // don't set left so that the right side is off screen
               f.left := min( getSystemMetrics(0)-f.width, f.left)
               f.top := t + (h/2 - f.height/2)
               // don't set top to less than zero
               f.top := max( f.top, 0 )
               // don't set top so that the bottom is off screen
               f.top := min( getSystemMetrics(1)-f.height, f.top)
            else
               f.autoCenter := true
            endif
*/
         catch (exception e)
            f.autoCenter := true
         endtry
         f.readModal()
         f.release()
      catch (exception e)
         msgbox(e.message,'Error #'+e.code,16)
      endtry
      return

   function siGetComputerName
      local cName
      cName = space(256)
      try
         if GetComputerName( cName, 256 )
            cName := trim(cName)
         else
            cName := 'Undetermined'
         endif
      catch( exception e)
         cName := 'Undetermined'
      endtry
      return cName

   function siGetDBEngine
      // Thanks to David Peters for this code (slightly modified
      // to show the BDE date as well as the version).
      local n, iVersionSize, sBuffer, iStart, c, sReturnString, sBDEPath, aDLLs
      try
         n = 0
         sReturnString = new string()

         local oReg
         oReg = new rmRegistry()
         oReg.SetMainKey(HKEY_LOCAL_MACHINE)
         oReg.SetSubKey(BDE_REG_KEY)
         sBdePath = oReg.GetValue("DLLPATH")
         release object oReg
/*
         r = new registry(HKEY_LOCAL_MACHINE, BDE_REG_KEY, false)
         sBDEPath = r.queryValue("DLLPATH")

         if ( sBDEPath.right(1) = ";" )
            sBDEPath = sBDEPath.left(sBDEPath.length-1)
         endif

         // Thanks to Vic McClung for this fix
         // the registry may have more than one path in it, i.e.:
         //    "C:\Progam Files\Common Files\Borland\BDE\;C:\IDAPI"
         // select the one with idapi32.dll
         aDLLs = tokenize(sBDEPath, ';')
         for i = 1 to aDLLs.size
            sBDEPath = aDLLs[i]+'\idapi32.dll'
            if file(sBDEPath)
               // if this is the one we're looking for
               exit
            endif
         next
*/

         sBDEPath = sBDEPath+[\idapi32.dll]
*        sBDEPath = "c:\windows\system32\user32.dll"
         // don't need to add chr(0) to a cstring
         iVersionSize = GetFileVersionInfoSizeA(sBDEPath, n)
         sBuffer = new STRING(SPACE(iVersionSize)+1)
         GetFileVersionInfoA(sBDEPath, 0, iVersionSize, sBuffer)

         iStart = AT("FileVersion", sBuffer)
         if ( EMPTY(iStart) )
            sReturnString = "Error"
         else
            iStart += 12
            isNull = false
            do while not isNull
               c = sBuffer.CharAt(iStart)
               if ( ASC(c) = 0 or iStart > 1024 )
                  isNull = true
               else
                  sReturnString += c
               endif
               iStart++
            enddo
         endif

        sReturnString = stuff(version(.89),at(":",version(.89))+2,0,sReturnString+' (')+')'
      catch (exception e)
         // if we fail to get the version, at least return the date
         sReturnString := version(.89)
         msgbox(e.message+chr(13)+'line '+e.lineNo+chr(13)+'file '+e.filename)
      endtry

      return (sReturnString)

   function siGetDDEServName
      return _app.ddeServiceName

   function siGetLangDriver
      return ldriver()

   function siGetLocalShare
      try

         local oReg

         oReg = new rmRegistry()

         oReg.SetMainKey(HKEY_LOCAL_MACHINE)
         oReg.SetSubKey(BDE_REG_KEY+[\Settings\SYSTEM\INIT])

         bLShare = oReg.GetValue([LOCAL SHARE])

         release object oReg

      catch (exception e)
         bLShare = 'Undetermined'
      endtry

      return bLShare

   function siGetMemoryInfo

      local m,nAvail,nTotal,lReturn

      lReturn = true

      try

         if type('GlobalMemoryStatusEx') # 'FP'
            extern clogical GlobalMemoryStatusEx(cptr) kernel32
         endif

         m = new _MEMORYSTATUSEX()

         lReturn := GlobalMemoryStatusEx(m)

*        m.showStructure()

         this.memoryLoad := ltrim(str(m.dwMemoryLoad()))+[%]
         this.PhysicalMemory := ltrim(transform(round(m.ullTotalPhys(),0)/1024, '999,999,999,999 Kb'))
         this.availableMemory := ltrim(transform(round(m.ullAvailPhys(),0)/1024, '999,999,999,999 Kb'))
         nTotal = m.ullTotalPageFile()
         nAvail = m.ullAvailPageFile()
         nUsage = round(100 - (100 * (nAvail / nTotal)),0)
         this.PageFileSize := ltrim(transform(round(m.ullTotalPageFile()/1024,0), '999,999,999,999 Kb'))
         this.PageFileUsage := ltrim(transform(nUsage, '999%'))

      catch (exception e)

         msgbox(e.message+chr(13)+'Line: '+e.lineno,'Error #'+e.code,16)

         this.PhysicalMemory := 'Undetermined'
         this.AvailableMemory := 'Undetermined'
         this.PageFileSize := 'Undetermined'
         this.PageFileUsage := 'Undetermined'

      endtry

      try
         m.release()
      catch (exception e)
      endtry
/*
      try
         m=new memoryStatus()
         GlobalMemoryStatus( m.value )
         this.PhysicalMemory := ltrim(transform(round(m.getMember('dwTotalPhys'),0)/1024, '999,999,999,999 kb'))
         this.availableMemory := ltrim(transform(round(m.getMember('dwAvailPhys'),0)/1024, '999,999,999,999 kb'))
         nTotal = m.getMember('dwTotalPageFile')
         nAvail = m.getMember('dwAvailPageFile')
         nUsage = round(100 - (100 * (nAvail / nTotal)),0)
         this.PageFileSize := ltrim(transform(round(m.getMember('dwTotalPageFile')/1024,0), '999,999,999,999 kb'))
         this.PageFileUsage := ltrim(transform(nUsage, '999%'))
      catch (exception e)
         msgbox(e.message+chr(13)+'Line: '+e.lineno,'Error #'+e.code,16)
         this.PhysicalMemory := 'Undetermined'
         this.AvailableMemory := 'Undetermined'
         this.PageFileSize := 'Undetermined'
         this.PageFileUsage := 'Undetermined'
      endtry
      try
         release object m
      catch (exception e)
      endtry
      release m
*/
      return

   function siGetRunTimeEngine
      return version(0)

   function siGetVideoInfo
      local h, n
      try
         this.screenRes := getsystemmetrics(0)+'x'+getsystemmetrics(1)
      catch (exception e)
         this.screenRes := 'Undetermined'
      endtry
      try
         // now get the display information
         h = CreateDC('DISPLAY', '','','')
         n = GetDeviceCaps( h, BITSPIXEL )
         do case
            case n > 16
               this.ColorDepth = 'True color'
            case n == 16
               this.ColorDepth = '16 bit (65536 colors)'
            case n == 8
               this.ColorDepth = '8 bit (256 color)'
            case n == 4
               this.ColorDepth = '4 bit (16 color)'
            case n == 2
               this.ColorDepth = 'Monochrome'
            otherwise
               this.ColorDepth = 'Undetermined'
         endcase
      catch (exception e)
         this.colorDepth := 'Undetermined'
      endtry
      release h, n
      return

   function siGetTableLevel
      local reg, cLevel, nLevel
      try
         local oReg
         oReg = new rmRegistry()
         oReg.SetMainKey(HKEY_LOCAL_MACHINE)
         oReg.SetSubKey(BDE_REG_KEY+"\Settings\DRIVERS\DBASE\TABLE CREATE")
         cLevel = oReg.GetValue("LEVEL")
         release object oReg
/*
         reg = new Registry(HKEY_LOCAL_MACHINE, ;
                        BDE_REG_KEY + ;
                        "\Settings\DRIVERS\DBASE\TABLE CREATE" )
         nLevel = reg.queryValue("LEVEL")
         if ( reg.error == 0 )
            cLevel = 'Level ' + nLevel
         else
            cLevel = 'Error reading registry'
         endif
*/
      catch (exception e)
         cLevel = 'Undetermined'
      endtry
*     release object reg
      return cLevel

   function getDbaseBuildNumber()
      return (int(val(stuff(version(0),1,at([b],version(0)),[]))))

   function getDbaseAllUsersHomePath()

      local cAllUsersHomePath

      cAllUsersHomePath = []

      do case
         case  class::getDbaseBuildNumber() > 2119                            // after 2.62
               cAllusersHomePath := left(_app.allUsersPath,(len(_app.allUsersPath)-4))
         otherwise                                                            // before 2.70
               cAllUsersHomePath := _dbwinhome
      endcase

      return (cAllUsersHomePath)

   function is_Process_Elevated()
      return (this.lAdministrator)

   function is_OS_W2K_or_Later()
      return (this.lW2KOn)

   function is_OS_64Bit()
      return (this.l64bit)

   function get_OS_Version()
      return (this.cOperatingSysVersion)

   function get_OS_VersionId()
      return (this.nOperatingSysVersionId)

   function get_OS_VersionBuild()
      return (this.nOperatingSysVersionBuild)

endclass

/*
class MEMORYSTATUS of structure
   super::addMember(TYPE_DWORD,'dwLength')        // sizeof(MEMORYSTATUS)
   super::addMember(TYPE_DWORD,'dwMemoryLoad')    // percent of memory in use
   super::addMember(TYPE_DWORD,'dwTotalPhys')     // bytes of physical memory
   super::addMember(TYPE_DWORD,'dwAvailPhys')     // free physical memory bytes
   super::addMember(TYPE_DWORD,'dwTotalPageFile') // bytes of paging file
   super::addMember(TYPE_DWORD,'dwAvailPageFile') // free bytes of paging file
   super::addMember(TYPE_DWORD,'dwTotalVirtual')  // user bytes of address space
   super::addMember(TYPE_DWORD,'dwAvailVirtual')  // free user bytes
endclass

class SYSTEM_INFO of Structure
   super::addMember( TYPE_WORD,   "wProcessor" )
   super::addMember( TYPE_WORD,   "wReserved" )
   super::addMember( TYPE_DWORD,  "dwPageSize" )
   super::addMember( TYPE_LPVOID, "lpMinimumApplicationAddress" )
   super::addMember( TYPE_LPVOID, "lpMaximumApplicationAddress" )
   super::addMember( TYPE_DWORD,  "dwActiveProcessorMask" )
   super::addMember( TYPE_DWORD,  "dwNumberOfProcessors" )
   super::addMember( TYPE_DWORD,  "dwProcessorType" )
   super::addMember( TYPE_DWORD,  "dwAllocationGranularity"  )
   super::addMember( TYPE_WORD,   "wProcessorLevel" )
   super::addMember( TYPE_WORD,   "wProcessorRevision" )
endclass

class RECT of Structure
   super::addMember( TYPE_LONG, 'left' )
   super::addMember( TYPE_LONG, 'top' )
   super::addMember( TYPE_LONG, 'right' )
   super::addMember( TYPE_LONG, 'bottom' )
endclass

class OSVERSIONINFO of structure
   super::addMember(TYPE_DWORD,'dwOSVersionInfoSize')
   super::addMember(TYPE_DWORD,'dwMajorVersion')
   super::addMember(TYPE_DWORD,'dwMinorVersion')
   super::addMember(TYPE_DWORD,'dwBuildNumber')
   super::addMember(TYPE_DWORD,'dwPlatformId')
   super::addMember(TYPE_STRING,'szCSDVersion', 128)
   this.setMember('dwOSVersionInfoSize',this.length())
endclass
*/

class AboutBoxForm of FORM
   set procedure to sysinfo.cc additive
   with (this)
      open = class::OPEN
      readModal = class::READMODAL
      scaleFontBold = false
      metric = 6  // Pixels
      height = 356.0
      left = 386.0
      top = 139.0
      width = 511.0
      text = "About"
      topMost = true
      mdi = false
      smallTitle = true
      sizeable = false
      maximize = false
      minimize = false
   endwith

   this.TABBOX1 = new TABBOX(this)
   with (this.TABBOX1)
      onSelChange = class::TABBOX1_ONSELCHANGE
      id = 123
      height = 22.0
      left = 0.0
      top = 334.0
      width = 511.0
      fontName = "Tahoma"
      fontSize = 8.0
      dataSource = 'ARRAY {"Program Info","System Info"}'
   endwith

   this.RECTANGLE1 = new RECTANGLE(this)
   with (this.RECTANGLE1)
      left = 7.0
      top = 9.0
      width = 496.0
      height = 277.0
      text = "Program Information"
      fontName = "Tahoma"
      fontSize = 8.0
      pageno = 0
   endwith

   this.TEXT1 = new LABELTEXT(this)
   with (this.TEXT1)
      height = 22.0
      left = 23.0
      top = 31.0
      width = 128.0
      value = "Application name:"
      pageno = 1
   endwith

   this.TEXT2 = new LABELTEXT(this)
   with (this.TEXT2)
      height = 22.0
      left = 23.0
      top = 53.0
      width = 128.0
      value = "Application version:"
      pageno = 1
   endwith

   this.TEXT3 = new LABELTEXT(this)
   with (this.TEXT3)
      height = 22.0
      left = 23.0
      top = 75.0
      width = 128.0
      value = "Application copyright:"
      pageno = 1
   endwith

   this.TEXT4 = new LABELTEXT(this)
   with (this.TEXT4)
      height = 22.0
      left = 23.0
      top = 97.0
      width = 128.0
      value = "Runtime engine:"
      pageno = 1
   endwith

   this.TEXT5 = new LABELTEXT(this)
   with (this.TEXT5)
      height = 22.0
      left = 23.0
      top = 119.0
      width = 128.0
      value = "Language driver:"
      pageno = 1
   endwith

   this.TEXT6 = new LABELTEXT(this)
   with (this.TEXT6)
      height = 22.0
      left = 23.0
      top = 141.0
      width = 128.0
      value = "Database engine:"
      pageno = 1
   endwith

   this.TEXT7 = new LABELTEXT(this)
   with (this.TEXT7)
      height = 22.0
      left = 23.0
      top = 163.0
      width = 128.0
      value = "Default table level:"
      pageno = 1
   endwith

   this.TEXT8 = new LABELTEXT(this)
   with (this.TEXT8)
      height = 17.0
      left = 23.0
      top = 185.0
      width = 128.0
      value = "Local share:"
      pageno = 1
   endwith

   this.TEXT9 = new LABELTEXT(this)
   with (this.TEXT9)
      height = 22.0
      left = 23.0
      top = 207.0
      width = 128.0
      value = "DDE service name:"
      pageno = 1
   endwith

   this.APPNAME = new LABELTEXT(this)
   with (this.APPNAME)
      height = 22.0
      left = 158.0
      top = 31.0
      width = 338.0
      value = ""
      pageno = 1
   endwith

   this.APPVERSION = new LABELTEXT(this)
   with (this.APPVERSION)
      height = 22.0
      left = 158.0
      top = 53.0
      width = 338.0
      value = ""
      pageno = 1
   endwith

   this.APPCOPYRIGHT = new LABELTEXT(this)
   with (this.APPCOPYRIGHT)
      height = 22.0
      left = 158.0
      top = 75.0
      width = 338.0
      value = ""
      pageno = 1
   endwith

   this.RUNTIMEENGINE = new LABELTEXT(this)
   with (this.RUNTIMEENGINE)
      height = 22.0
      left = 158.0
      top = 97.0
      width = 338.0
      value = ""
      maxLength = 36
      pageno = 1
   endwith

   this.LANGDRIVER = new LABELTEXT(this)
   with (this.LANGDRIVER)
      height = 22.0
      left = 158.0
      top = 119.0
      width = 338.0
      value = ""
      pageno = 1
   endwith

   this.DBENGINE = new LABELTEXT(this)
   with (this.DBENGINE)
      height = 22.0
      left = 158.0
      top = 141.0
      width = 338.0
      value = ""
      maxLength = 34
      pageno = 1
   endwith

   this.DEFAULTTABLELEVEL = new LABELTEXT(this)
   with (this.DEFAULTTABLELEVEL)
      height = 22.0
      left = 158.0
      top = 163.0
      width = 338.0
      value = ""
      pageno = 1
   endwith

   this.LOCALSHARE = new LABELTEXT(this)
   with (this.LOCALSHARE)
      height = 22.0
      left = 158.0
      top = 185.0
      width = 338.0
      value = ""
      pageno = 1
   endwith

   this.DDENAME = new LABELTEXT(this)
   with (this.DDENAME)
      height = 22.0
      left = 158.0
      top = 207.0
      width = 338.0
      value = ""
      pageno = 1
   endwith

   this.PUSHBUTTON1 = new PUSHBUTTON(this)
   with (this.PUSHBUTTON1)
      onClick = {;form.close()}
      height = 24.0
      left = 219.0
      top = 298.0
      width = 72.0
      text = "Ok"
      default = true
      fontName = "Tahoma"
      fontSize = 8.0
      pageno = 0
   endwith

   this.TEXT30 = new LABELTEXT(this)
   with (this.TEXT30)
      height = 22.0
      left = 23.0
      top = 30.0
      width = 167.0
      value = "Processors:"
      pageno = 2
   endwith

   this.TEXT22 = new LABELTEXT(this)
   with (this.TEXT22)
      height = 22.0
      left = 23.0
      top = 51.0
      width = 167.0
      value = "Computer name:"
      pageno = 2
   endwith

   this.TEXT23 = new LABELTEXT(this)
   with (this.TEXT23)
      height = 22.0
      left = 23.0
      top = 73.0
      width = 167.0
      value = "System version:"
      pageno = 2
   endwith

   this.TEXT24 = new LABELTEXT(this)
   with (this.TEXT24)
      height = 22.0
      left = 23.0
      top = 150.0
      width = 167.0
      value = "Screen resolution:"
      pageno = 2
   endwith

   this.TEXT25 = new LABELTEXT(this)
   with (this.TEXT25)
      height = 22.0
      left = 23.0
      top = 172.0
      width = 167.0
      value = "Screen color depth:"
      pageno = 2
   endwith

   this.TEXT26 = new LABELTEXT(this)
   with (this.TEXT26)
      height = 22.0
      left = 23.0
      top = 194.0
      width = 167.0
      value = "Total physical memory:"
      pageno = 2
   endwith

   this.TEXT27 = new LABELTEXT(this)
   with (this.TEXT27)
      height = 22.0
      left = 23.0
      top = 216.0
      width = 145.0
      value = "Available memory:"
      maxLength = 26
      pageno = 2
   endwith

   this.TEXT28 = new LABELTEXT(this)
   with (this.TEXT28)
      height = 22.0
      left = 23.0
      top = 238.0
      width = 167.0
      value = "Page file size:"
      pageno = 2
   endwith

   this.TEXT29 = new LABELTEXT(this)
   with (this.TEXT29)
      height = 22.0
      left = 23.0
      top = 261.0
      width = 167.0
      value = "Page file usage:"
      pageno = 2
   endwith

   this.PROCESSOR = new LABELTEXT(this)
   with (this.PROCESSOR)
      height = 22.0
      left = 168.0
      top = 31.0
      width = 280.0
      value = ""
      pageno = 2
   endwith

   this.COMPUTERNAME = new LABELTEXT(this)
   with (this.COMPUTERNAME)
      height = 22.0
      left = 168.0
      top = 51.0
      width = 280.0
      value = ""
      pageno = 2
   endwith

   this.SYSTEMVERSION = new LABELTEXT(this)
   with (this.SYSTEMVERSION)
      height = 22.0
      left = 168.0
      top = 73.0
      width = 310.0
      value = ""
      pageno = 2
   endwith

   this.SCREENRES = new LABELTEXT(this)
   with (this.SCREENRES)
      height = 22.0
      left = 168.0
      top = 150.0
      width = 280.0
      value = ""
      pageno = 2
   endwith

   this.COLORDEPTH = new LABELTEXT(this)
   with (this.COLORDEPTH)
      height = 22.0
      left = 168.0
      top = 172.0
      width = 280.0
      value = ""
      pageno = 2
   endwith

   this.PHYSICALMEMORY = new LABELTEXT(this)
   with (this.PHYSICALMEMORY)
      height = 22.0
      left = 168.0
      top = 194.0
      width = 280.0
      value = ""
      pageno = 2
   endwith

   this.AVAILABLEMEMORY = new LABELTEXT(this)
   with (this.AVAILABLEMEMORY)
      height = 22.0
      left = 168.0
      top = 216.0
      width = 280.0
      value = ""
      pageno = 2
   endwith

   this.PAGEFILESIZE = new LABELTEXT(this)
   with (this.PAGEFILESIZE)
      height = 22.0
      left = 168.0
      top = 238.0
      width = 280.0
      value = ""
      pageno = 2
   endwith

   this.PAGEFILEUSAGE = new LABELTEXT(this)
   with (this.PAGEFILEUSAGE)
      height = 22.0
      left = 168.0
      top = 261.0
      width = 280.0
      value = ""
      pageno = 2
   endwith

   this.ENTRYFIELD1 = new LABELTEXT(this)
   with (this.ENTRYFIELD1)
      height = 22.0
      left = 23.0
      top = 127.0
      width = 167.0
      value = "Native system type:"
      pageno = 2
   endwith

   this.NATIVESYSTEMTYPE = new LABELTEXT(this)
   with (this.NATIVESYSTEMTYPE)
      height = 22.0
      left = 168.0
      top = 127.0
      width = 310.0
      value = ""
      pageno = 2
   endwith

   this.ENTRYFIELD2 = new LABELTEXT(this)
   with (this.ENTRYFIELD2)
      height = 22.0
      left = 23.0
      top = 88.0
      width = 167.0
      value = "System version number:"
      pageno = 2
   endwith

   this.SYSTEMVERSIONID = new LABELTEXT(this)
   with (this.SYSTEMVERSIONID)
      height = 22.0
      left = 168.0
      top = 88.0
      width = 310.0
      value = ""
      pageno = 2
   endwith

   this.ENTRYFIELD3 = new LABELTEXT(this)
   with (this.ENTRYFIELD3)
      height = 22.0
      left = 23.0
      top = 104.0
      width = 167.0
      value = "System version build:"
      pageno = 2
   endwith

   this.SYSTEMVERSIONBUILD = new LABELTEXT(this)
   with (this.SYSTEMVERSIONBUILD)
      height = 22.0
      left = 168.0
      top = 104.0
      width = 310.0
      value = ""
      pageno = 2
   endwith


   function open
      this.mdi=false
      this.init()
      return super::readModal()

   function readModal
      this.init()
      return super::readModal()

   function Init
      with (this)
         appCopyright.value = SysInfo.appCopyright
         appName.value = SysInfo.appName
         appVersion.value = SysInfo.appVersion
         availableMemory.value = SysInfo.availableMemory
         colorDepth.value = SysInfo.colorDepth
         computerName.value = SysInfo.computerName
         dbEngine.value = SysInfo.dbEngine
         ddeName.value = SysInfo.ddeName
         defaultTableLevel.value = SysInfo.defaultTableLevel
         langDriver.value = SysInfo.langDriver
         localShare.value = SysInfo.localShare
         systemVersion.value = SysInfo.cOperatingSysVersion
         systemVersionId.value = ltrim(str(SysInfo.get_OS_versionId(),3,1))
         systemVersionBuild.value = ltrim(str(SysInfo.get_OS_versionBuild()))
         nativeSystemType.value = iif(SysInfo.l64bit,[64-bit],[32-bit])
         pageFileSize.value = SysInfo.pageFileSize
         pageFileUsage.value = SysInfo.pageFileUsage
         physicalMemory.value = SysInfo.physicalMemory
         processor.value = SysInfo.processor
         runtimeEngine.value = SysInfo.runtimeEngine
         screenRes.value = SysInfo.screenRes
      endwith
      return

   function TABBOX1_onSelChange
      form.pageNo=this.curSel
      if this.curSel==2
         form.rectangle1.text='System Information'
      else
         form.rectangle1.text='Program Information'
      endif
      return

endclass

class LabelText(oParent) of entryfield(oParent) custom
   with(this)
      colorNormal = 'BtnText/BtnFace'
      borderStyle = 3
      mousePointer = 1
      when = {||false}
   endwith
endclass

// ----------------------------------------------------------------------------
function tokenize(cString, cDelimiter)
   local aTokens; aTokens = new array()
   local nCntr; nCntr = 1
   local nPos
   if pcount() < 2
      cDelimiter = ' '
   endif
   do while .t.
      nPos = at(cDelimiter, cString)
      if nPos == 0
         nPos = len(cString)+1
         if nPos == 1
            exit
         endif
      endif
      aTokens.add(1)
      aTokens[nCntr] = left(cString, nPos-1)
      nCntr = nCntr+1
      if nPos > len(cString)
         exit
      endif
      cString = substr(cString, nPos+1)
   enddo
   return aTokens

