/*
   -----------------------------------------------------------------------------
   File...: UserBDEAlias.cc
   Author.: Ken Mayer
   Date...: April, 2021
   
   Description:
   A custom class to emulate some of the code in BDEAlias.cc, which writes
   to the ini file for the BDE. The User BDE Alias code should write/read/
   etc. to/from the application's .ini file. This modifies the .ini file
   used by either dBASE or your own executable.
   
   The advantage to a User BDE Alias is that you don't have to deal with
   the UAC's tendency to require the user be logged in as administrator,
   and the alias doesn't require modifying the BDE's entries at all.
   
   Dependencies: 
       :dUFLP:ini.cc, which relies on 
       :dUFLP:stringex.cc
   
   Methods contained here:
   
      init
         Initializes the class, it is called automatically, you don't
         need to do anything.
         
      release
         Code cleanup. See example below.
   
      CreateAlias
      
         Create the entries in the .ini file being used for the application
         needed for a User BDE Alias. Requires three parameters 
         noted below. If an alias by the database name provided 
         already exists, it will be overwritten, so the driver and
         options values will be modified with new information, rather
         than creating a new section in the .ini file.
         
         Parameters:
            DatabaseName
            Driver Name
            Options -- must include PATH
         
         Driver Names:
            Native Drivers:
               DBASE
               FOXPRO
               PARADOX
               ASCIIDRV (text files?)
            SQL Link Drivers:
               DB2
               INFORMIX
               INTRBASE
               MSACCESS
               MSSQL
               ORACLE
               SYBASE
         The first four are native to dBASE the last are SQL Link driver 
         names.
         
         For a dBASE database (or other local driver) you must provide an 
         option that includes the path, for other databases you need to 
         typically provide more information, besides the path:
            Server Name:
            User Name:
            SQLQRYMODE:
            Path:
            ODBC DSN:
            DB2 DSN: 

         Simplest: 
            "PATH:C:\dUFLP 2021"
         
         More Complex:
         Each of these must be a string, so nested in quotes, each option
         must be separated by a semicolon (;). An example might be:
            "PATH:'C:\ProgramData\MyCompany\MyApplication\Tables\employees.gdb'; SERVER NAME: ''; USER NAME: ''; SQLQRYMODE: 'LOCAL'"
         Note the use of single and double quotes ...
         
         The path can be with or without quotes, but if you have other options
         it may be a good idea ...
         
         (Note: All the above is from The dBASE Book, Part 1, Ch. 5, pages 62-63)
         
         It should be noted the code to create the alias does not perform
         any error checking on the options provided. If you made a mistake,
         however, you can simply call the method again with corrected options,
         and the code will overwrite what is there.
         
         The CreateAlias method must close and restart the program, because
         the .ini file needs to be re-loaded because the new addition
         of the User BDE Alias is not in memory, and there doesn't appear to be
         any other way to do this. 
         
      DatabaseDir
         Returns the path from the Options for the User BDE Alias as defined
         in the .ini file.
      
      DatabaseFound
         Returns a logical/boolean value (true or false) if the database
         name is found in the .ini file.
      
      DeleteAlias
         Returns a logical/boolean value based on success -- removes
         the entries needed from the .ini file, if it fails, it returns
         a false value. A typo in the name of the database alias can
         cause a fail ...
      
      ListAliases
         This method returns an array of alias names. If the optional
         parameter (boolean/logical) is passed with a value of
         true, the array will be a two column array, column 1 will contain
         the alias names, column 2 will contain the paths to where the
         tables are.
         
      ListTables
         Returns an array of the table names contained in the database. 
      
   Examples:

      set procedure to :dUFLP:BDEUserAlias.cc additive
      b = new UserBDEAlias()

      // CreateAlias
      ? b.CreateAlias( "MYALIAS", "DBASE" "PATH:C:\TEST" ) // true/false
      
      // DatabaseDir
      ? b.DatabaseDir( "MYALIAS" ) // should return path

      // DatabaseFound
      ? b.databaseFound( "MYALIAS" ) // true/false results
      
      // DeleteAlias
      ? b.DeleteAlias( "MYALIAS" ) // true/false results
      
      // ListAliases:
      aList = b.ListAliases()      // just the alias names
      for i = 1 to aList.size
          ? aList[ i ]
      next
      
      aList2 = b.ListAliases(true) // two column array with 
                                   // alias names and paths
      for i = 1 to aList2.size/2 // divide by two 
          ? aList2[ i, 1] + " -- " + aList2[ i, 2 ]
      next
      
      // ListTables:
      aTables = b.ListTables( "MyAlias" )
      for i = 1 to aTables.size
          ? aTables[ i ]
      next
      
      // Cleanup:
      b.release()
      b := null
      close procedure :dUFLP:UserBDEAlias.cc
   -----------------------------------------------------------------------------
*/

class UserBDEAlias 
   class::init() // initialize the class by setting up the .ini code

   /*
      Initialize by opening ini.cc, creating an instance of the
      ini object that uses _app.iniFile.
   */
   function init
      set procedure to :dUFLP:ini.cc
      this.ini = new ini( _app.iniFile )
   return
   // End of Method: init
   
   /*
      A bit of cleanup code, not 100% required, but a good idea
      to call it when done.
   */
   function release
      this.ini.release() // releases the stringex.cc class
      this.ini := null
      close procedure :dUFLP:ini.cc
      release object this
   return null
   // End of Method: release

   /*
      CreateAlias
      Creates a User BDE Alias, has four parameters, the first
      three are required:
         cDatabase = name of database
         cDriver   = database driver to be used (see comments above)
         cOptions  = options, at a minimum must contain the path for
                     the location of the database, ex:
                         "Path:C:\dUFLP 2021"
         bMsg      = if the developer provides a value of TRUE,
                     a small dialog box appears after a successful
                     creation of the User BDE Alias letting them
                     know. If the developer provides a value of FALSE
                     no message appears. The default is FALSE
                     (meaning the developer must provide a TRUE
                     value for the message). 
                     
      The code creates entries in the .ini file for the application
      under the [UserBDEAliases] section and then creates the 
      appropriate database section. 
      
      If the database already exists, it will update it (useful if the
      path needs changing, for example). 
      
      ** WARNING ** There is no error check for the contents of the
      contents of the options.
      
      ** WARNING 2 ** When this method completes, the application
      will close and restart. This is because until this happens,
      the new User BDE Alias, while in the appropriate .ini file,
      is not in MEMORY, and the .ini file needs to be reloaded.
      This is inconvenient, but not horribly. A coder who uses
      this to create the alias for a deployed application can
      do something like:
         set procedure to :dUFLP:UserBDEAlias.cc
         oUBA = new UserBDEAlias()
         if not oUBA.DatabaseFound( "MyAlias" )
            oUBA.CreateAlias( "MyAlias", "DBASE", "PATH:F:\dUFLP 2021", true )
         endif
         oUBA.release()
         close procedure :dUFLP:UserBDEAlias.cc
         
      The program will close and restart and the alias will exist. From that
      point the application will test (as above), find that the database
      exists, and not try to re-create it. 
   */
   function CreateAlias( cDatabase, cDriver, cOptions, bMsg )
      local bComplete, aEntries, nEntries, bExists, nPos, i, nEntry, cEntry, bLF
      // deal with parameters, need them all ...
      if pCount() < 3
         msgbox( "Error, need all three parameters: cDatabase, cDriver, cOptions",;
                 "User BDE Alias error", 16 )
         return false
      endif
      if pCount == 3 // we have only three parameters, so the
                     // developer didn't pass the fourth:
         bMsg = false
      endif
      
      // get a list of current aliases, we don't want to add
      // the alias a second time if it already exists:
      aEntries = this.ini.GetSection("UserBDEAliases")
      nEntries= aEntries.size // in case we need to loop through
      
      // does it exist? We can't really use the DatabaseFound()
      // method here, because we need to know the "position"
      // in the list if it does exist:
      bExists = false
      nPos = -1
      for i = 1 to aEntries.size
          // if database name is contained in the entry:
          if cDatabase.toLowerCase() $ aEntries[ i ].toLowerCase()
             bExists := true
             nPos = i // position in array
             exit
          endif
      next // i
      
      // if it doesn't exist, we need to use the last entry
      // which starts with a number:
      if not bExists 
         nPos = aEntries.size // last entry
      endif
      
      // get the numeric value of the entry which is the
      // number to the left of the equal sign -- we don't
      // need any fancy string manipulation as the val()
      // function just grabs the numeric value and returns
      // the number:
      nEntry = val( aEntries[ nPos ] ) // value of the entry

      // if the entry does not exist we want to add 1 to the
      // LAST entry value, and use that:
      if not bExists
         nEntry++
      endif
      // make it a character value:
      cEntry = ""+nEntry
      
      // note: no validation is being done on driver or options,
      // if the user got it wrong, the database entry won't work.
      
      // add the entries to the .ini file:
      cSection = cDatabase
      // add/update the entry under the UserBDEAlias section
      this.ini.SetValue("UserBDEAliases", cEntry, cSection)
      // create new section or overwrite old one:
      this.ini.SetValue( cSection, "Driver", cDriver )
      this.ini.SetValue( cSection, "Options", cOptions )

      // message the user that the database has been created,
      // but they need to exit dBASE and restart, to re-load the
      // .ini file (sheesh).
      if bMsg
         cLF = chr(13)+chr(10) // line feed for msgbox text
         msgbox( "User BDE Alias '" + cSection + "' has been created. " + cLF +;
                 "You need to exit the application and re-start it for the " + cLF +;
                 "alias to be available to your application code." + cLF + cLF +;
                 "After you click the 'OK' button the program will exit and restart.",;
                 "User BDE Alias Created Successfully", 64 )
      endif
      // restart the executable
      run( true, _app.EXEName)
      // quit the current instance of the same executable ...
      quit
   return true
   // End of Method: CreateAlias
   
   /*
      DeleteAlias
      Delete an alias, both from the list of alias under the section
      [UserBDEAlias] in the .ini file, but also the complete database
      section, which includes the driver and options. Not as complex as 
      it sounds.
   */
   function DeleteAlias( cDatabase )
      local aEntries, bComplete
      bComplete = false
      // error check:
      if pCount() == 0
         msgbox( "You need to pass the name of the database to delete!",;
                 "User BDE Alias error", 16 )
         return false
      endif
      
      // does it even exist?
      if this.DatabaseFound( cDatabase )
         // since it must to be here, now, we need to delete
         // the section and the key ... for example, say we 
         // have under the [UserBDEAliases] section header:
         //    8=Dummy
         // And then below we have this section:
         //    [Dummy]
         //    Driver=dBASE
         //    Options=Path: F:\Nowhere\Tables
         // We should delete the section AND the item under 
         // the section header.
         // I fear we're probably going to have to loop through
         // an array to get the numeric value we need ...:
         aEntries = this.ini.GetSection( "UserBDEAliases" )
         for i = 1 to aEntries.size
             if lower( cDatabase) $ lower( aEntries[ i ] )
                cEntry = aEntries[ i ]
               exit
             endif // lower ...
         next // endfor
         // having that, we need to get the numeric value to the
         // left of the equal sign:
         nEntry = val( cEntry ) // value of the entry
         // then turn it back to a string:
         cEntry = ""+nEntry

         // with that:
         this.ini.DeleteEntry( "UserBDEAliases", cEntry ) // delete the entry
         // then we should be able to delete the section for the database:
         this.ini.DeleteSection( cDatabase )
         
         // if we got here:
         bComplete := true
      endif
   return bComplete
   // End of Method: DeleteAlias
   
   /*
      DatabaseFound
      This method will return a logical value (true or false) if the database
      name as passed is found in the User BDE Aliases in the .ini file.
   */
   function DatabaseFound( cDatabase )
      local aEntries, bFound, i
      bFound = false // default
      
      // error check:
      if pCount() == 0
         msgbox( "You need to pass the name of the database to look for!",;
                 "User BDE Alias error", 16 )
         return false
      endif

      // get a list of database names
      aEntries = this.ini.GetSection( "UserBDEAliases" )
      // find the one we're looking for
      for i = 1 to aEntries.size
          if lower( cDatabase) $ lower( aEntries[ i ] )
             bFound := true
             exit
          endif // lower ...
      next // endfor
   return bFound
   // End of Method: DatabaseFound
   
   /* 
      DatabaseDir
      Returns the path for a User BDE Alias if appropriate -- 
      if a path does not exist will return an empty string,
      although if the database name is not passed, you will
      see an error.
   */
   function DatabaseDir( cDatabase )
      local aEntries, cPath, cOptions
      
      // error check:
      if pCount() == 0
         msgbox( "You need to pass the name of the database!",;
                 "User BDE Alias error", 16 )
         return "User error -- pass the name of the database"
      endif

      // first check to see if the database exists:
      if this.DatabaseFound( cDatabase )
         // go to the section for the database, and get the options value:
         cOptions = this.ini.GetValue( cDatabase, "Options" )
         // look for "path" in it:
         if "path" $ lower( cOptions )
            // some of the option strings may have semicolons, which means
            // multiple options, we only want the path:
            if ";" $ cOptions
               // extract the path from the options in case needed, 
               cOptions = substr( cOptions, at( "path:", lower(cOptions) ), at( ";", cOptions ) -1 )
            endif
            // extract only the path (remove the word "path" and the 
            // colon
            cPath = right( cOptions, len(cOptions ) - at( ":", cOptions ) )
            cPath = ltrim( cPath ) // in case of extra spaces
            // we also need to remove quotes (single or double):
            if "'" $ cPath 
               cPath = StripChar( cPath, "'" )
            endif
            if '"' $ cPath 
               cPath = StripChar( cPath, '"' )
            endif
         else
            cPath = "Path not found in ini file"
         endif // path not found
      else
         cPath = "Database not found, cannot return a path."
      endif // database not found
   return cPath
   // End of Method: DatabaseDir
   
   /*
      ListAliases
      This simple method will list all aliases that are 
      in the .ini file under the UserBDEAlias section heading.
      This returns an array of aliases. 
      
      The parameter bPaths defaults to false (and is optional -- if left out we
      deal with setting it to false),  if it is passed as true, then a 
      second column will be added to the array, which will include the paths to 
      the individual aliases as stored in the Options string.
   */
   function ListAliases
      local aEntries, i, nEntries, nES_Pos, cEntry, cSection, cPath
      parameter bPaths
      if type( "bPaths" ) # "L" // not logical, which includes "U" for undefined
         bPaths = false
      endif
      
      // get the alias names in the UserBDEAliases section:
      aEntries = this.ini.GetSection( "UserBDEAliases" )
      nEntries= aEntries.size
      for i = 1 to nEntries
          cEntry = aEntries[ i ]
          // we need to replace the alias with 
          // just the name, which is the text to the right
          // of the equal sign:
          nES_Pos = at( "=", cEntry )
          cEntry = right( cEntry, len( cEntry ) - nES_Pos )
          aEntries[ i ] = cEntry
      next
      
      // Here's where it gets more interesting, we need
      // to, if the parameter bPaths is true, add a column
      // to the array, then go back to the ini file and 
      // get the paths ... 
      if bPaths
         aEntries.grow(2) // add a column
         
         // loop through the array
         for i = 1 to nEntries
             cSection = aEntries[ i, 1 ] // row, first column
             cOptions = this.ini.GetValue( cSection, "Options" )
             if "path" $ lower( cOptions )
                // use the databaseDir() method to get the path:
                cPath = this.databaseDir( cSection )
                // add to array's second column
                aEntries[ i, 2 ] = cPath
             else
                aEntries[ i, 2 ] = "Path not found in ini file."
             endif
         next
      endif
   return aEntries
   // End of Method: ListAliases
   
   /*
      ListTables
      If the database exists, return an array of table names contained in 
      the database. Should work for any type of database, as we're using 
      the database object's getSchema() method. 
   */
   function ListTables( cDatabase )
      local aTables, dData
      
      // error check:
      if pCount() == 0
         msgbox( "You need to pass the name of the database!",;
                 "User BDE Alias error", 16 )
         aTables = new array()
         aTables.add( "User error -- pass the name of the database to the method!" )
         return aTables
      endif

      // only try this if the database exists:
      if this.DatabaseFound( cDatabase )
         // create a database object and activate it
         dData = new database()
         dData.databaseName := cDatabase
         dData.active       := true
         // get list of tables
         aTables             = dData.getSchema( "Tables" )
         // cleanup
         dData.active       := false
         release object dData
         dData              := null
      else // not found
         aTables = new array()      
         aTables.add( "Database not found, cannot list tables." )
      endif
      
   return aTables
   // End of Method: ListTables
   
endclass

// pulled from :dUFLP:StringEX to deal with removing quotes
// from strings in the DatabaseDir method.
FUNCTION StripChar( cStrArg, cStrip, lWord )
/*
   -----------------------------------------------------------------
   Programmer..: Steve Saville
   Date........: January 15, 2002
   Notes.......: Strips selected character(s) out of alphanumeric
                 string (like perhaps, removing hyphens from a 
                 vehicle tag number: C-123-456 would become C123456)
                 (uppercase/lowercase doesn't matter)
   Written for.: dB2K
   Rev. History: 01/15/2002 -- Copied StripND and modified to remove
                 any character(s) included in the second parameter
   Calls.......: None
   Usage.......: stringex.StripChar(<cStrArg>,<cStrip>)
   Example.....: ? _app.String.StripChar("C-123-456","C-",false)
                 Result: "123456"
                 ?_app.string.stripchar("JustATestString","Test",false)
                 Result: "JuAring"
                 ?_app.string.stripchar("JustATestString","Test",true)
                 Result: "JustAString"
   Returns.....: character string stripped of desired character(s)
   Parameters..: cStrArg = Character memvar containing alphanumeric
                           string
                 cStrip  = Character(s) to strip from cStrArg
                 lWord   = If true, strips the exact word or phrase
                           providing an exact match is found in 
                           cStrArg. If false, strips every character 
                           which has a match from cStrArg
  ------------------------------------------------------------------
*/
   local cRetVal, nCount, cChar
   cRetVal = ""
   if lWord = false
      for nCount = 1 to len( cStrArg )
            cChar = substr(cStrArg,nCount,1)
            if not upper(cChar) $upper(cStrip)
               cRetVal = cRetVal+cChar
            endif
      endfor
   else
      cRetVal = iif(at(upper(cStrip),upper(cStrArg)) = 0,cStrArg,;
      left(cStrArg,at(upper(cStrip),upper(cStrArg)) -1)+;
      right(cStrArg,(len(cStrArg) - at(upper(cStrip),;
      upper(cStrArg)) - len(cStrip) + 1)))
   endif
   
RETURN cRetVal
// EoM: StripChar()
