//  -- File:  GRAPHEX.cc

CLASS GRAPHEX

/* ------------------------------------------------------------------------------------

   Description:


      Creates simplified extended graphics line and box functions

      routines that can be called virtually in place of the '?' command

      system-wide variable allows programmer to turn on/off support
         without changing code - depending on customer desire and system.



   Programmers:

      John Marshall
      john.marshall@mainstreetprinceton.com


   History:

      October 2005:    Began set up and testing .CC
      November 2005:   Added text bounded by lines with justification
                       Added constants and settings to control use




   Instantiation:


      _app.graphex = 'Y'   (any other character turns off the use)

      set procedure to Graphex.Cc additive
      GPr = new G()

     ** Be sure to have the following set in the PLUS.INI

           [CommandWindow]
           ResultsPaneFont=Courier New, 9		**
           InputPaneFont=Courier New, 10     **

           **  This just needs to be a font supporting graphics characters  **

           [CommandSettings]
           ldriver = DB437US0


   Functions:


      example:
      GPr.sbox(nOffset,nHeight,nWidth,sText,mJust)

      print a single line box with 'nOffset' margin, nHeight and nWidth
      using justification 'R', 'C' or 'L'


   FUNCTION LIST:

   Function     Description                         Parameters
----------------------------------------------------------------------

   sBox:        Single Line Box       [Offset,Height,Width,Text,Just]
   dBox:        Double Line Box       [Offset,Height,Width,Text,Just]

   SeSbLine:	 Single Bottom, Single Edge         [offset, width]
   DeSbLine:    Single Bottom, Double Edge         [offset, width]

   SeDbLine:    Double Bottom, Single Edge         [offset, width]
   DeDbLine:    Double Bottom, Double Edge         [offset, width]
                                                   
   SeStLine:    Single Top, Single Edge            [offset, width]
   DeStLine:    Single Top, Double Edge            [offset, width]

   SeDtLine:    Double Top, Single Edge            [offset, width]
   DeDtLine:    Double Top, Double Edge            [offset, width]

   SesLine:    Single Edged, Single line		      [offset, width]
   DesLine:		Double Edged, Single Line	         [offset, width]

   SedLine:    Single Edged, Double Line           [offset, width]
   DedLine:    Double Edged, Double Line           [offset, width]

   DLine:		Double Line									[offset, width]	
   SLine:		Single Line									[offset, width]	

   SeTLine:		Single Edged Text Line					[offset, width, text, just]
   DeTLine:		Double Edged Text Line              [offset, width, text, just]

   SE:			Empty Line with Single Edges			[offset, width]
   DE:			Empty Line with Double Edges        [offset, width]
              
-------------------------------------------------------------------------*/ 

//  **  Define variables for graphics use/non-use

//      if user specifies use of extended graphics,
//      otherwise use standard ascii characters

IF _app.graphex = 'Y'

   //  Basic Horizontal and Vertical Characters

	this.slinechar = 196      // single line character
   this.dlinechar = 205      // double line character
   this.sendchar  = 179      // single end character (vertical)
   this.dendchar  = 186      // double end character (vertical)

   //  Line Ends "T" (right and left)
   this.sEndsLinel = 195     // single vertical end with single Hor. line (left)
   this.sEndsLiner = 180     // single vertical end with single Hor. line (right)
   this.sEnddLinel = 198     // single vertical end with double Hor. line (left)
   this.sEnddLiner = 181     // single vertical end with double Hor. line (left)

   this.dEndsLinel = 199     // double vertical end with single Hor. line (left)
   this.dEndsLiner = 182     // double vertical end with single Hor. line (left)
   this.dEnddLinel = 204     // double vertical end with double Hor. line (left)
   this.dEnddLiner = 185     // double vertical end with double Hor. line (left)
   
   //  single line box corners

   this.Sul       = 218      // single upper left
   this.Sur       = 191
   this.Sll       = 192
   this.Slr       = 217

   this.sel       = 195
   this.del       = 199
   this.ser       = 180
   this.der       = 182

   //  single edge with double top/bottom box corners (left/right)

   this.sedtul    = 213
   this.sedtur    = 184
   this.sedbll    = 212
   this.sedblr    = 190

   //  double edge with single top/bottom box corners (left/right)

   this.destul    = 214
   this.destur    = 183
   this.desbll    = 211
   this.desblr    = 189

   // double line box corners

   this.Dul       = 201      // double upper left
   this.Dur       = 187
   this.Dll       = 200
   this.Dlr       = 188


      //  Define other characters:  vsLine, vdLine (vert single, double)
      //  sTee, dTee (singe/double "T") etc 


ELSE    //  User selects NOT to use extended graphics

	this.slinechar = 45	// dash
   this.dlinechar = 45
   this.sendchar  = 32	 
   this.dendchar  = 32  // space

   this.sEndsLinel = 32
   this.sEndsLiner = 32
   this.sEnddLinel = 32
   this.sEnddLiner = 32

   this.dEndsLinel = 32
   this.dEndsLiner = 32
   this.dEnddLinel = 32
   this.dEnddLiner = 32

   this.Sul       = 32
   this.Sur       = 32
   this.Sll       = 32
   this.Slr       = 32

   this.sel       = 32
   this.del       = 32 
   this.ser       = 32
   this.der       = 32

   this.sedtul    = 32
   this.sedtur    = 32
   this.sedbll    = 32
   this.sedblr    = 32

   this.destul    = 32
   this.destur    = 32
   this.desbll    = 32
   this.desblr    = 32

   this.Dul       = 32
   this.Dur       = 32
   this.Dll       = 32
   this.Dlr       = 32

                    		//  124 = |

ENDIF

//**********************************************************  Function Body

   Function SE    //  Single Edged (no text)
      parameter nOffset,nWidth

       ? space(nOffset) + chr(this.sendchar) + space(nWidth-2) + chr(this.sendchar)

   return

   function Se2CtLine
      parameter nOffset, nWidth

      mMiddle = floor(nWidth/2)
      ? space(nOffset) + chr(this.sul)
      for i = 4 to nWidth
         ?? chr(this.slinechar)
         if i = mMiddle + 2
            ?? chr(194)
         endif
      next i
      ?? chr(this.sur)

      return

   function Se2cTxLine      //  **  Single Edged 2 Column Text Line
      parameter nOffset, nWidth, sText1, sText2, mJust

      try
         mText1 = ltrim(str(sText1))
         mText2 = ltrim(str(sText2))
      catch (Exception e)
         mText1 = sText1
         mText2 = sText2
      endtry

      if substr(mJust,1,1)$'LRlr' //and len(mJust>1)
         mMargin = space(val(substr(mJust,2,1)))
      else 
         mMargin = space(0)
      endif

      mMiddle = floor(nWidth/2)

      if nWidth > 1

            _rmargin = 132
            _wrap    = false
            _pageno = 1
            _plineno = 0
            _ppitch  ="condensed"
         
         //  **  Data Line  ************
         ? space(nOffset) + chr(this.sendchar)

         if upper(mJust) = 'R'

            ?? right(space(132) + mText1 + mMargin, mMiddle-1)
            ?? chr(this.sendchar)

            if nWidth/2 = floor(nWidth/2)
               ?? right(space(132) + mText2 + mMargin, mMiddle-2)
            else
               ?? right(space(132) + mText2 + mMargin, mMiddle-1)
            endif

         else if upper(mJust) = 'C'

            mLen1 = len(mText1)
            mLen2 = len(mText2)
            nSide1 = floor(((mMiddle-1)-mLen1)/2)
            nSide2 = floor(((mMiddle-1)-mLen2)/2)
            ?? space((mMiddle-1)-(nSide1 + mLen1)) + left(mText1 + space(132), (nSide1 + mLen1))
            ?? chr(this.sendchar)

            if nWidth/2 = floor(nWidth/2)
               ?? space((mMiddle-2)-(nSide2 + mLen2)) + left(mText2 + space(132), (nSide2 + mLen2))
            else
               ?? space((mMiddle-1)-(nSide2 + mLen2)) + left(mText2 + space(132), (nSide2 + mLen2))
            endif

         else

            ?? left(mMargin + mText1 + space(132),mMiddle-1)
            ?? chr(this.sendchar)
            if nWidth/2 = floor(nWidth/2)
               ?? left(mMargin + mText2 + space(132),mMiddle-2)
            else
               ?? left(mMargin + mText2 + space(132),mMiddle-1)
            endif

         endif

         ?? chr(this.sendchar)

      endif
      
      return

   function Se2CbLine
      parameter nOffset, nWidth

      mMiddle = floor(nWidth/2)
      ? space(nOffset) + chr(this.sll)
      for i = 4 to nWidth
         ?? chr(this.slinechar)
         if i = mMiddle + 2
            ?? chr(193)
         endif
      next i
      ?? chr(this.slr)
      
      return



   Function DE    //  Double Edged (no text)
      parameter nOffset,nWidth

      ? space(nOffset) + chr(this.dendchar) + space(nWidth-2) + chr(this.dendchar)

   return


   function SLine   // Single Line (no edges)
      parameter nOffset,nWidth

      ? space(nOffset)
      for i = 1 to nWidth
         ?? chr(this.slinechar)
      next i
      
      return

   function oSLine   // Single Line (no edges)
      parameter nOffset,nWidth

      ?? space(nOffset)
      for i = 1 to nWidth
         ?? chr(this.slinechar)
      next i
      
      return

   function DLine   // Double Line (no edges)
      parameter nOffset,nWidth

      ? space(nOffset)
      for i = 1 to nWidth
         ?? chr(this.dlinechar)
      next i
      
      return

   function oDLine   // Offset Double Line (no edges) (follows earlier line)
      parameter nOffset,nWidth

      ?? space(nOffset)
      for i = 1 to nWidth
         ?? chr(this.dlinechar)
      next i
      
      return

   function sBox    //  Single Line Box (title opt)
      parameter nOffset, nHeight, nWidth, sText, mJust

      if type("nOffset") == "U"
         nOffset = 0
      endif
      if type("nHeight") == "U"
         nHeight = 3
      endif
      if type("sText") == "U"
         sText = ""
      endif
      if type("mJust") == "U"  or not mJust$"LCR"
         mJust = 'C'  // ** enforce L,C,R
      endif
      if substr(mJust,1,1)$'LR' and len(mJust>1)
         mMargin = space(val(substr(mJust,2,1)))
      else 
         mMargin = space(0)
      endif

      //  test for string or number
      try
         mText = ltrim(str(sText))
      catch (Exception e)
         mText = sText
      endtry

      if type("nWidth") == "U" or nWidth < ( len(mText) +4 )
         nWidth  = len(mText) + 4
      endif

            _lmargin = nOffset
            _rmargin = 132
            _wrap    = false
            _pageno = 1
            _plineno = 0
            _ppitch  ="condensed"
          
         //  **  Top Bar  **************
         ? space(nOffset) + chr(this.Sul)
         for i = 3 to nWidth
            ?? chr(this.SlineChar)
         next i
         ?? chr(this.Sur)


         //  Deal with height here, place single line to bottom
         if nHeight > 3
            mTop = floor((nHeight-3)/2)
            mBot = (nHeight - 3) - mTop
            for i = 0 to mtop
               this.Se(nOffset, nWidth)
            next i
         endif


         //  **  Data Line  ************
         ? space(nOffset) + chr(this.sendchar)
         if upper(mJust) = 'R'
            ?? right(space(132) + mText + mMargin,nWidth-2)
         else if upper(mJust) = 'C'
            mLen = len(mText)
            nSide = floor(((nWidth-2)-mLen)/2)
            ?? space((nWidth-2)-(nSide+mLen)) + left(mText + space(132), (nSide + mLen))
         else
            ?? left(mMargin + mText + space(132),nWidth-2)
         endif
         ?? chr(this.sendchar)


         //  Deal with height here, place single line to bottom
         if nHeight > 3
            for i = 0 to mbot
               this.Se(nOffset, nWidth)
            next i
         endif


         //  **  Bottom Bar  ***********
         ? space(nOffset) + chr(this.Sll)
         for i = 3 to nWidth
            ?? chr(this.SlineChar)
         next i
         ?? chr(this.Slr)

            
      return

   function SeTLine      //  **  Single Edged Text Line
      parameter nOffset,nWidth, sText, mJust

      try
         mText = ltrim(str(sText))
      catch (Exception e)
         mText = sText
      endtry

      if nWidth > 1

            _rmargin = 132
            _wrap    = false
            _pageno = 1
            _plineno = 0
            _ppitch  ="condensed"
         
         //  **  Data Line  ************
         ? space(nOffset) + chr(this.sendchar)
         if upper(mJust) = 'R'
            ?? right(space(132) + mText,nWidth-2)
         else if upper(mJust) = 'C'
            mLen = len(mText)
            nSide = floor(((nWidth-2)-mLen)/2)
            ?? space((nWidth-2)-(nSide+mLen)) + left(mText + space(132), (nSide + mLen))
         else
            ?? left(mText+space(132),nWidth-2)
         endif

         ?? chr(this.sendchar)

      Endif
      

      
      return

   function DeTLine      //  **  Double Edged Text Line
      parameter nOffset,nWidth, sText,mJust

      try
         mText = ltrim(str(sText))
      catch (Exception e)
         mText = sText
      endtry

      if nWidth > 1

            _rmargin = 132
            _wrap    = false
            _pageno = 1
            _plineno = 0
            _ppitch  ="condensed"
         
         //  **  Data Line  ************
         ? space(nOffset) + chr(this.dendchar)

         if upper(mJust) = 'R'
            ?? right(space(132) + mText,nWidth-2)
         else if upper(mJust) = 'C'
            mLen = len(mText)
            nSide = floor(((nWidth-2)-mLen)/2)
            ?? space((nWidth-2)-(nSide+mLen)) + left(mText + space(132), (nSide + mLen))
         else
            ?? left(mText+space(132),nWidth-2)
         endif

         ?? chr(this.dendchar)

      Endif
      
      return

   function SeSbLine        //  **  Single edge, single bottom line
      parameter nOffset,nWidth

      if nWidth > 1

            _lmargin = nOffset
            _rmargin = 132
            _wrap    = false
            _pageno = 1
            _plineno = 0
            _ppitch  ="condensed"
         
         //  **  Top Bar  **************
         ? space(nOffset) + chr(this.sll)
         for i = 3 to nWidth
            ?? chr(this.sLineChar)
         next i
         ?? chr(this.slr)
       Endif
      return

   function SeDtLine      //  **  Single edge, Double top line
      parameter nOffset,nWidth
      
      if nWidth > 1

            _lmargin = nOffset
            _rmargin = 132
            _wrap    = false
            _pageno = 1
            _plineno = 0
            _ppitch  ="condensed"
         
         //  **  Top Bar  **************
         ? space(nOffset) + chr(this.sedtul)
         for i = 3 to nWidth
            ?? chr(this.dLineChar)
         next i
         ?? chr(this.sedtur)
       endif

      
      return

   function SeStLine     //  **  Single edge, single top line
      parameter nOffset,nWidth
      
      if nWidth > 1

            _lmargin = nOffset
            _rmargin = 132
            _wrap    = false
            _pageno = 1
            _plineno = 0
            _ppitch  ="condensed"
         
         //  **  Top Bar  **************
         ? space(nOffset) + chr(this.sul)
         for i = 3 to nWidth
            ?? chr(this.slinechar)
         next i
         ?? chr(this.sur)
       endif
      return
      
      return

   function DesLine     //  **  Double Edge Single line
      parameter nOffset,nWidth

      if nWidth > 1

            _lmargin = nOffset
            _rmargin = 132
            _wrap    = false
            _pageno = 1
            _plineno = 0
            _ppitch  ="condensed"
         
         //  **  Top Bar  **************
         ? space(nOffset) + chr(this.dEndsLinel)
         for i = 3 to nWidth
            ?? chr(this.sLineChar)
         next i
         ?? chr(this.dEndsLiner)
       Endif
      
      return

   function SedLine    //  **  Single Edge Double line
      parameter nOffset,nWidth
      
      if nWidth > 1

            _lmargin = nOffset
            _rmargin = 132
            _wrap    = false
            _pageno = 1
            _plineno = 0
            _ppitch  ="condensed"
         
         //  **  Top Bar  **************
         ? space(nOffset) + chr(this.sEnddLinel)
         for i = 3 to nWidth
            ?? chr(this.dLineChar)
         next i
         ?? chr(this.sEnddLiner)
       Endif

      
      return

   function SeDbLine       //  **  Single Edge Double bottom line
      parameter nOffset,nWidth
     
      if nWidth > 1

            _lmargin = nOffset
            _rmargin = 132
            _wrap    = false
            _pageno = 1
            _plineno = 0
            _ppitch  ="condensed"
         
         //  **  Top Bar  **************
         ? space(nOffset) + chr(this.sedbll)
         for i = 3 to nWidth
            ?? chr(this.dLinechar)
         next i
         ?? chr(this.sedblr)
       Endif
      
      return

   function DedLine      //  **  Double Edge Double line
      parameter nOffset,nWidth
      
      if nWidth > 1

            _lmargin = nOffset
            _rmargin = 132
            _wrap    = false
            _pageno = 1
            _plineno = 0
            _ppitch  ="condensed"
         
         //  **  Top Bar  **************
         ? space(nOffset) + chr(this.denddlinel)
         for i = 3 to nWidth
            ?? chr(this.dLineChar)
         next i
         ?? chr(this.denddLiner)
       Endif


      
      return

   function SesLine    //  **  Single Edge Single line
      parameter nOffset,nWidth
      
      if nWidth > 1

            _lmargin = nOffset
            _rmargin = 132
            _wrap    = false
            _pageno = 1
            _plineno = 0
            _ppitch  ="condensed"
         
         //  **  Top Bar  **************
         ? space(nOffset) + chr(this.sel)
         for i = 3 to nWidth
            ?? chr(this.slinechar)
         next i
         ?? chr(this.ser)
       Endif


      
      return

   function DeDtLine    //  **  Double Edge, Double top line
      parameter nOffset,nWidth
      
      if nWidth > 1

            _lmargin = nOffset
            _rmargin = 132
            _wrap    = false
            _pageno = 1
            _plineno = 0
            _ppitch  ="condensed"
         
         //  **  Top Bar  **************
         ? space(nOffset) + chr(this.dul)
         for i = 3 to nWidth
            ?? chr(this.dLineChar)
         next i
         ?? chr(this.dur)
       Endif
      return

   function DeStLine    //  **  Double edge Single top line
      parameter nOffset,nWidth
      
      if nWidth > 1

            _lmargin = nOffset
            _rmargin = 132
            _wrap    = false
            _pageno = 1
            _plineno = 0
            _ppitch  ="condensed"
         
         //  **  Top Bar  **************
         ? space(nOffset) + chr(this.destul)
         for i = 3 to nWidth
            ?? chr(this.slinechar)
         next i
         ?? chr(this.destur)
       Endif

      
      return

   function DeSbLine     //  **  Double edge Single bottom line
      parameter nOffset,nWidth
      
      if nWidth > 1

            _lmargin = nOffset
            _rmargin = 132
            _wrap    = false
            _pageno = 1
            _plineno = 0
            _ppitch  ="condensed"
         
         //  **  Top Bar  **************
         ? space(nOffset) + chr(this.desbll)
         for i = 3 to nWidth
            ?? chr(this.slinechar)
         next i
         ?? chr(this.desblr)
       Endif
 
      
      return

   function DeDbLine    //  **  Double edge/bottom line
      parameter nOffset,nWidth
      
      if nWidth > 1

            _lmargin = nOffset
            _rmargin = 132
            _wrap    = false
            _pageno = 1
            _plineno = 0
            _ppitch  ="condensed"
         
         //  **  Bottom Bar  **************
         ? space(nOffset) + chr(this.dll)
         for i = 3 to nWidth
            ?? chr(this.dLineChar)
         next i
         ?? chr(this.dlr)
       Endif

      return

   function dBox
      parameter nOffset,nHeight,nWidth, sText,mJust

      //  **  Double box with text

      if type("nOffset") == "U"
         nOffset = 0
      endif
      if type("nHeight") == "U"
         nHeight = 3
      endif
      if type("sText") == "U"
         sText = ""
      endif
      if type("mJust") == "U"  or not mJust$"LCR"
         mJust = 'C'  // ** enforce L,C,R
      endif

      if substr(mJust,1,1)$'LR' and len(mJust>1)
         mMargin = space(val(substr(mJust,2,1)))
      else 
         mMargin = space(0)
      endif

      try
         mText = ltrim(str(sText))
      catch (Exception e)
         mText = sText
      endtry

      if type("nWidth") == "U" or nWidth < ( len(mText) +4 )
         nWidth  = len(mText) + 4
      endif

            _lmargin = nOffset
            _rmargin = 132
            _wrap    = false
            _pageno = 1
            _plineno = 0
            _ppitch  ="condensed"
          
         //  **  Top Bar  **************
         ? space(nOffset) + chr(this.Dul)
         for i = 3 to nWidth
            ?? chr(this.DlineChar)
         next i
         ?? chr(this.Dur)


         //  Deal with height here, place single line to bottom
         if nHeight > 3
            mTop = floor((nHeight-3)/2)
            mBot = (nHeight - 3) - mTop
            for i = 0 to mtop
               this.De(nOffset, nWidth)
            next i
         endif


         //  **  Data Line  ************
         ? space(nOffset) + chr(this.Dendchar)
         if upper(mJust) = 'R'
            ?? right(space(132) + mText + mMargin,nWidth-2)
         else if upper(mJust) = 'C'
            mLen = len(mText)
            nSide = floor(((nWidth-2)-mLen)/2)
            ?? space((nWidth-2)-(nSide+mLen)) + left(mText + space(132), (nSide + mLen))
         else
            ?? left(mMargin + mText + space(132),nWidth-2)
         endif
         ?? chr(this.Dendchar)


         //  Deal with height here, place single line to bottom
         if nHeight > 3
            for i = 0 to mbot
               this.De(nOffset, nWidth)
            next i
         endif


         //  **  Bottom Bar  ***********
         ? space(nOffset) + chr(this.Dll)
         for i = 3 to nWidth
            ?? chr(this.DlineChar)
         next i
         ?? chr(this.Dlr)

            
      return


endclass
