// Metric: 0 - Chars or 6 - Pixels only
/*
   --------------------------------------------------------------------
   PassVerify.cc
   Programmer.: Ken Mayer
   Date.......: August 22, 2001
   Written for: dB2K, release 2 or later (Source Alias)
   Notes......:  
   This is container that has TWO PasswordEntry controls on it,
   which will allow the user to add some functionality to
   their password routines, basically allowing a new user
   to enter a password twice, the second time to verify
   the entry used the first time.

   Usage notes:
      As some systems require passwords to be case sensitive,
      this has been taken into account here. Rather than
      using the caseSensitive property of the password
      control itself, which is designed for a user actually
      logging in to a system, I have added a caseSensitive
      property to the passVerify control. 

      To use this:

         set procedure to :dUFLP:PassVerify.cc additive

      In the form designer, drag an instance of the
      passverify container to the form surface from
      the "Custom" tab of the Component Palette.

      If you need the control to be case sensitive, then
      in the form's onOpen event:
         this.passVerify1.caseSensitive := true

      If you want to set a minimum size for a password
      (i.e., many systems require a password be at least
      four characters), use the password control's minLength
      property:
          this.passVerify1.passWordEntry1.minLength := 4
          this.passVerify1.passWordEntry2.minLength := 4

       ======================================================
       You could override the onOpen event of this control,
       as well, by making sure the container (passVerify1)
       has focus, in the inspector, click on the "Events"
       tab, and find "onOpen". When you get to the "onOpen"
       event, click the tool. You will be asked if you wish
       to override the event. Click the 'Yes' button. In
       the source code editor, make sure you enter:
           PassVerify:onOpen() // call original onOpen event

       Then you can set properties for the control:
           this.caseSensitive := true
           this.passWordEntry1.minLength := 4
           this.passWordEntry2.minLength := 4
       ======================================================

      The code for validation is set up in such a way that
      while the password controls are not specifically datalinked
      to a field in your table, you can assign the values before
      saving the row, for example, in the rowset's canSave
      event:
          f = this.parent.parent // form reference
          this.fields["MyPassword"].value := ;
                     f.passVerify.passwordEntry1.value

      The simpler option would be to use the dataLink 
      property for one of the two passwordEntry controls 
      (you don't need to datalink both of them).

      Since you can't exit the container until both values entered
      match, there's no other validation required.

   ---------------
   Note that this uses the source alias "dUFLP", and
   from the dUFLP routines, it uses password.cc
   --------------------------------------------------------------------
*/

#define MSK "*" // Mask Character

class PassVerify( oParent ) of CONTAINER( oParent ) custom
set procedure to :duflp:password.cc additive
   with (this)
      left = 3.2857
      top = 5.3182
      width = 34.4286
      height = 3.6364
      metric = 0	// Chars
      borderStyle = 3 // None
   endwith

   this.TEXTLABEL1 = new TEXTLABEL(this)
   with (this.TEXTLABEL1)
      height = 1
      left = 0.7143
      top = 0.3182
      width = 14.2857
      text = "Enter Password:"
      pageno = 0
      alignHorizontal = 2	// Right
   endwith

   this.TEXTLABEL2 = new TEXTLABEL(this)
   with (this.TEXTLABEL2)
      height = 1
      left = 1
      top = 2.0909
      width = 14
      text = "Verify Password:"
      pageno = 0
      alignHorizontal = 2	// Right
   endwith

   this.PASSWORDENTRY1 = new PASSWORDENTRY(this)
   with (this.PASSWORDENTRY1)
      height = 1
      left = 16.1429
      top = 0.2727
      width = 15
      pageno = 0
   endwith

   this.PASSWORDENTRY2 = new PASSWORDENTRY(this)
   with (this.PASSWORDENTRY2)
      height = 1
      left = 16.1429
      top = 1.9091
      width = 15
      pageno = 0
   endwith

   function onDesignOpen( lFromPalette )
      if lFromPalette and form.metric == 6 // Pixels
         this.height := 80
         this.width  := 241
         this.PasswordEntry1.height :=  22
         this.PasswordEntry1.left   := 113
         this.PasswordEntry1.top    :=   6
         this.PasswordEntry1.width  := 105
         this.PasswordEntry1.height :=  22
         this.PasswordEntry1.left   := 113
         this.PasswordEntry1.top    :=  42
         this.PasswordEntry1.width  := 105
         this.TextLabel1.height     :=  22
         this.TextLabel1.left       :=   5
         this.TextLabel1.top        :=   7
         this.TextLabel1.width      := 100
         this.TextLabel2.height     :=  22
         this.TextLabel2.left       :=   5
         this.TextLabel2.top        :=  46
         this.TextLabel2.width      :=  98
      endif
   return

   function OnOpen
      // When container opens, let's set properties for
      // the two password fields:

      // since this is designed for creating a password,
      // we won't track the number of attempts to 
      // enter a "valid" entry
      this.passwordEntry1.numTriesAllowed := 0
      this.passwordEntry2.numTriesAllowed := 0
      // this is really not necessary, but ...
      this.passwordEntry1.validCheckRequired := false
      this.passwordEntry2.validCheckRequired := false
      // The mask character:
      this.passwordEntry1.maskChar := MSK
      this.passwordEntry2.maskChar := MSK

      // set the code location for the second password field, so
      // that in its onLostFocus, we check the values of both
      // of the entryfields:
      this.passwordEntry2.onLostFocus := class::Validate

      // Set the caseSensitive property here:
      this.caseSensitive = false // default 

   function Validate
      local cValue1, cValue2

      // this is called in the onLostFocus event of the
      // second password field
      cValue1 = this.parent.passwordEntry1.value.rightTrim()
      cValue2 = this.value.rightTrim()

      if NOT this.parent.caseSensitive // passverify.caseSensitive
         cValue1 := cValue1.toUpperCase()
         cValue2 := cValue2.toUpperCase()
      endif

      // compare them:
      if NOT ( cValue1 == cValue2 )

         // all is NOT well, the two don't match:
         msgbox( "The second password entry does not match the first.",;
                 "Try again!", 16 )
         this.parent.passwordEntry1.setFocus()
      endif

      // if we get here, there's nothing else that needs
      // to be done. The password value can be saved
      // by the developer in the rest of their code ...

endclass
/*
    End of file: PassVerify.cc
*/
